


# from SpectrumCommon.Const.GB import *
# from SpectrumCommon.Const.DP import *
from SpectrumCommon.Const.TB import *
from SpectrumCommon.Modvars.GB.GBDefs import *
from SpectrumCommon.Modvars.TB.TBIndices import *



tb_patient_shape = lambda d, **kwargs : gb_shape(d,'1D float',
                                                  dim=['tb_patient'], 
                                                  indices=[tb_patient_indices], **kwargs)

tb_patient_year_shape = lambda d, **kwargs : gb_shape(d,'2D float',
                                                  dim=['tb_patient',  gb_year_dim], 
                                                  indices=[tb_patient_indices, gb_year_indices], **kwargs)

tb_hh_shape = lambda d, **kwargs : gb_shape(d,'1D float',
                                                  dim=['tb_household'], 
                                                  indices=[tb_household_indices], **kwargs)

tb_hh_year_shape = lambda d, **kwargs : gb_shape(d,'2D float',
                                                  dim=['tb_household',  gb_year_dim], 
                                                  indices=[tb_household_indices, gb_year_indices], **kwargs)

tb_art_shape = lambda d, **kwargs : gb_shape(d,'2D float',
                                                  dim=['tb_art_severity', 'tb_art_age'], 
                                                  indices=[tb_art_severity_indices, tb_art_age_indices], **kwargs)

tb_art_year_shape = lambda d, **kwargs : gb_shape(d,'3D float',
                                                  dim=['tb_art_severity','tb_art_age',  gb_year_dim], 
                                                  indices=[tb_art_severity_indices, tb_art_age_indices,  gb_year_indices], **kwargs)


tb_hrg_shape = lambda d, **kwargs : gb_shape(d,'1D float',
                                                  dim=['tb_highriskgroup'], 
                                                  indices=[tb_hrg_indices], **kwargs)

tb_hrg_year_shape = lambda d, **kwargs : gb_shape(d,'2D float',
                                                  dim=['tb_highriskgroup',  gb_year_dim], 
                                                  indices=[tb_hrg_indices, gb_year_indices], **kwargs)
                                                  
tb_child_adult_year_shape = lambda d, **kwargs : gb_shape(d,'2D float',
                                                  dim=['tb_child_adult',  gb_year_dim], 
                                                  indices=[tb_child_adult_indices, gb_year_indices], **kwargs)

tb_pulmonary_year_shape = lambda d, **kwargs : gb_shape(d,'2D float',
                                                  dim=['tp_pulmonary',  gb_year_dim], 
                                                  indices=[tb_pulmonary_indices, gb_year_indices], **kwargs)

tb_pulmonary_child_adult_year_shape = lambda d, **kwargs : gb_shape(d,'3D float',
                                                  dim=['tp_pulmonary', 'tb_child_adult',  gb_year_dim], 
                                                  indices=[tb_pulmonary_indices, tb_child_adult_indices, gb_year_indices], **kwargs)

tb_case_resistance_shape = lambda d, **kwargs : gb_shape(d,'3D float',
                                        dim=['tb_cases', 'tb_resistance_ages',  gb_year_dim], 
                                        indices=[tb_cases_indices, tb_resist_age_indices, gb_year_indices], **kwargs)

tb_impact_year_shape = lambda d, **kwargs : gb_shape(d,'2D float',
                                                  dim=['tp_impact',  gb_year_dim], 
                                                  indices=[tb_impact_indices, gb_year_indices], **kwargs)

tb_resbounds_year_shape = lambda d, **kwargs : gb_shape(d,'2D float',
                                                  dim=['tp_result_bounds',  gb_year_dim], 
                                                  indices=[tb_result_bound_indices, gb_year_indices], **kwargs)




TB_modvar_shapes = {

        TB_BaseYearTag: gb_int_shape('Base year is the starting notification year', range=[TB_Min_First_Year, TB_Max_First_Year], note='Not editable'),
        TB_DataSourceTag : gb_str_shape('Source of TB data'),
        TB_UseOldFailsafeTag: gb_bool_shape('Only true for old 2021 projections',  note='Not editable'), 
        TB_UseOldIPTag: gb_bool_shape('only true for old 2022 projections',  note='Not editable'),


        TB_HistNotifStartYearTag : gb_int_shape('Historical notification start year'),
        TB_HistNotifTag          : gb_shape('Historical notification','dict', innerdict={  
                                    key:gb_shape(f'Historical data for {key}',
                                                         "1D float", 
                                                         dim=['tb_hist_year']
                                                         ) for key in TB_NOTIF_DB_KEYS
                                  }),
        TB_HistNotiDistributionTag : gb_shape('Historical notification distribution','dict', innerdict={  
                                    key:gb_shape(f'Historical data for {key}',
                                                         "1D float", 
                                                         dim=['tb_hist_year']
                                                         ) for key in TB_HistNotiDistributionTag
                                  }),
                                            
        TB_ScreeningAlgMapTag              : gb_shape('Screening algorithm mapping','dict', innerdict={            
                                                       i : gb_shape(i, 
                                                         "1D float", 
                                                         dim=['tb_sens_spec'],
                                                         indices=[{'Sensitivity':0, 'Specificity':1}]) for i in tb_screening_options 

                                            }),

                            


        # From the wireframes
        TB_PulmonaryAgeTag                 : gb_shape('Selected patient screening algorithm','1D int',
                                                       dim=['tb_patient'], 
                                                       indices = [tb_patient_indices],
                                                       has_default=False),
        
        TB_PrevalenceTag                   : tb_patient_year_shape('Prevalence by patient groups'),
        TB_PatientScreenSensitivityTag     : tb_patient_shape('Patient screening sensitivity'),
        TB_PatientScreenSpecificityTag     : tb_patient_shape('Patient screening specificity'), 
        TB_PatientScreenAlgOptionsTag      : gb_shape('Patient screening algorithm options','2D str',
                                                       dim=['tb_patient', 'tb_screening_options'], 
                                                       indices = [tb_patient_indices, tb_screening_options],
                                                       has_default=False),

        TB_PatientScreenAlgSelectedTag     : gb_shape('Selected patient screening algorithm','1D str',
                                                       dim=['tb_patient'], 
                                                       indices = [tb_patient_indices],
                                                       options = tb_patient_screening_options,
                                                       has_default=False),

        TB_NumClinicallyAssessedTag        : tb_patient_year_shape('Patient number clinically assessed'),
        TB_NumReferredTag                  : tb_patient_year_shape('Patient number referred'),
        TB_NumScreenedToReferTag           : tb_patient_year_shape('Patient number screened top refer'),
        TB_PercTrueCasesTag                : tb_patient_year_shape('Patient percent true cases'),

        TB_PatientDiagCaseValuesTag            : gb_shape('Patient diagnostic values', '3D float',
                                                  dim=['tb_patient', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_patient_indices, [], []]),
        TB_PatientDiagCaseIDsTag               : gb_shape('Patient diagnostic IDs', '3D int',
                                                  dim=['tb_patient', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_patient_indices, [], []]),
        TB_PatientDiagCaseMethodsTag         : gb_shape('Patient diagnostic Method IDs', '2D int',
                                                  dim=['tb_patient', 'tb_dynamic'], 
                                                  indices = [tb_patient_indices, [], []]),
        TB_PatientDiagCaseSensitivityTag       : gb_shape('Patient diagnostic Sesnitivity', '3D float',
                                                  dim=['tb_patient', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_patient_indices, [], []]),
        TB_PatientDiagCaseSpecificityTag       : gb_shape('Patient diagnostic Specificity', '3D float',
                                                  dim=['tb_patient', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_patient_indices, [], []]),

        TB_PatientDiagSensitivityTag       : tb_patient_shape('Patient diagnosistic sensitivity'),
        TB_PatientDiagSpecificityTag       : tb_patient_shape('Patient diagnosistic specificity'),
        TB_NumDiagnosedPTBTag              : tb_patient_year_shape('Patient number diagnosed PTB'),
        TB_NumConfirmedPTBTag              : tb_patient_year_shape('Patient number confirmed PTB'),
        TB_NumDiagnosedTBTag               : tb_patient_year_shape('Patient number diagnosed TB'),
        TB_NumInitTreatmentTag             : tb_patient_year_shape('Patient number initializad on treatment'),

        TB_PatientScreeningOutputTag       : gb_shape('Patient screening output', '2D float',
                                                       dim=['tb_patient', 'tb_screening',  gb_year_dim], 
                                                       indices=[tb_patient_indices, tb_screening_indices, gb_year_indices]),
        TB_PatientDiagnosticOutputTag      : gb_shape('Patient screening output', '2D float',
                                                       dim=['tb_patient', 'tb_diagnosis',  gb_year_dim], 
                                                       indices=[tb_patient_indices, tb_diagnosis_indices, gb_year_indices]),
        TB_PatientTargetPopTag             : gb_shape('Patient screening output', '2D float',
                                                       dim=['tb_patient', 'tb_patient_targ_pop',  gb_year_dim], 
                                                       indices=[tb_patient_indices, tb_patient_targ_pop_indices, gb_year_indices]),
            

        # provider households
        
        # Household 
        ## Household demographics
        ### Demographics
        TB_HHSizeTag         : gb_float_shape('Average household size'),
        TB_HHPropU5Tag       : gb_float_shape('Proportion of household that is u5 '),
        TB_HHProp5_14Tag     : gb_float_shape('Proportion of household that is 5-14 yrs'),
        TB_HHCasesinIndexTag : gb_float_shape('Average number of TB cases in household with at least one case'),

        ###  TB infection characteristics
        TB_HHPropU5LTBITag : gb_year_shape('Proportion of u5s with TB infection in household with TB infection'),
        TB_HHPropO5LTBITag : gb_year_shape('Proportion of other household members with TB infection in household with active case'),

        ####  TB disease characteristics
        TB_HHPropU5TBTag  :  gb_year_shape('Proportion of u5s with TB disease in household with TB disease'),
        TB_HHPropU5PTBTag :  gb_year_shape('Proportion of u5s with TB disease in household with PTB disease'),  
        TB_HHPropO5TBTag  :  gb_year_shape('Proportion of other household members with TB disease in household with TB disease'),
        TB_HHPropO5PTBTag : gb_year_shape('Proportion with pulmonary TB, PTB'), 

        ##     Index cases
        TB_HHNumNewBactPosTag : gb_year_shape('Number of new bacteriologically positive TB cases (adult)'),
        TB_HHNumNewBactNegTag : gb_year_shape('Number of new bacteriologically negative TB cases (adult)'),

        ## household contact tracing coverage
        TB_HHPIndexCasesScreenedTag : gb_year_shape('Proportion of households of index cases screened/evaluated'),
        TB_HHPropBACnScreenedTag    : gb_year_shape('Proportion of contacts of new bacteriologically negative cases screened/evaluated'),
        
        # ## Tab Household contacts by age
        TB_HHContacts                : gb_shape('Household contacts', '3D float',
                                                 dim=['tb_household', 'tb_contact',  gb_year_dim], 
                                                 indices=[tb_household_indices, tb_contact_indices, gb_year_indices]),
        
        # ## Tab Household screeningtb_contact_indices
        # ### Household proportion
        TB_HHProportionTag           : tb_hh_year_shape('Household proportion'),

        # ### Household sensitivity amd specificity
        TB_HHScreenSensitivityTag : tb_hh_shape('Household sensitivity'), # np.full((TB_HHAgeLen), 0.90),
        TB_HHScreenSpecificityTag : tb_hh_shape('Household specificity'), # np.full((TB_HHAgeLen), 0.99),  
        TB_HHScreenAlgOptionsTag  : gb_shape('Household screening algorithm options','2D str',
                                              dim=['tb_household', 'tb_screening_options'], 
                                              indices = [tb_household_indices, tb_screening_options],
                                              has_default=False),
        TB_HHScreenAlgSelectedTag : gb_shape('Selected household screening algorithm','1D str',
                                              dim=['tb_household'], 
                                              indices = [tb_household_indices],
                                              options = tb_household_screening_options,
                                              has_default=False),


        # ### Household screening outputs
        TB_HHNumScreenedTag           : tb_hh_year_shape('Household number of screened'),
        TB_HHNumReferredTag           : tb_hh_year_shape('Household number referred '),
        TB_HHNumScreenedToReferTag    : tb_hh_year_shape('Household number screened to refer '),
        TB_HHPercTrueCasesTag         : tb_hh_year_shape('Household percent of true cases'),     

        # ## Tab Household diagnostic evaluation
        # ###  TB Case type split by treatment and sputum status

        TB_HHDiagCaseValuesTag            : gb_shape('Household diagnostic values', '3D float',
                                                  dim=['tb_household', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_household_indices, [], []]),
        TB_HHDiagCaseIDsTag               : gb_shape('Household diagnostic IDs', '3D int',
                                                  dim=['tb_household', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_household_indices, [], []]),
        TB_HHDiagCaseMethodsTag         : gb_shape('Household diagnostic Method IDs', '2D int',
                                                  dim=['tb_household', 'tb_dynamic'], 
                                                  indices = [tb_household_indices, [], []]),
        TB_HHDiagCaseSensitivityTag       : gb_shape('Household diagnostic Sesnitivity', '3D float',
                                                  dim=['tb_household', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_household_indices, [], []]),
        TB_HHDiagCaseSpecificityTag       : gb_shape('Household diagnostic Specificity', '3D float',
                                                  dim=['tb_household', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_household_indices, [], []]),
        # ### Household sensitivity and specificity

        TB_HHDiagSensitivityTag : tb_hh_shape('Household diagnostic sensitivity'),
        TB_HHDiagSpecificityTag : tb_hh_shape('Household diagnostic specificity'),

        # ### Diagnosis outputs
        TB_HHNumDiagnosedPTBTag       : tb_hh_year_shape('Household diagnosed PTB'),             
        TB_HHNumConfirmedPTBTag       : tb_hh_year_shape('Household confirmed PTB'),  

        # ## Tab Household disease and treatment
        TB_HHPropLinkedTreatTag       : tb_hh_year_shape('Household proportion linked to treatment'),           
        TB_HHNumInitTreatmentTag      : tb_hh_year_shape('Household number initialized on treatment'),         
        
        # ## Tab Household prevention
        TB_HHPrvntInActiveEligibleTag  : tb_hh_year_shape('Household prevention inactive eligible'),   
        TB_HHPrvntReceivePTPrevTag     : tb_hh_year_shape('Household prevention received PTB prevention'),   
        TB_HHPropPrvntPresumpTag       : tb_hh_year_shape('Household proportion prevention presumed'),
        TB_HHPropPrvntTestedTag        : tb_hh_year_shape('Household proportion prevention tested'), 
        
        TB_HHPrvntMethodsTag           : gb_shape('Household prevention method values', '2D float',
                                             dim=['tb_household', 'tb_dynamic_id'], 
                                             indices=[tb_household_indices, []]),   
        TB_HHPrvntMethodIDsTag         : gb_shape('Household prevention method IDs', '2D int',
                                             dim=['tb_household', 'tb_dynamic_id'], 
                                             indices=[tb_household_indices, []],
                                             options=tb_test_ids),

        TB_HHPropEligiblePrvntLinkTag  : tb_hh_year_shape('Household proportion linked to treatment'),   
        TB_HHNumberOnTPTTag            : tb_hh_year_shape("Household Number initiated on TB preventative treatment"),
        TB_HHTPTCompleteRateTag        : tb_hh_year_shape("Household TB prevention completion rate"),
        TB_HHNumberCompleteTPTTag      : tb_hh_year_shape("Household Number complete preventative treatment"),

        # ## Tab Household output
        TB_HHScreeningOutputTag  : gb_shape('Household screening output', '3D float',
                                             dim=['tb_household', 'tb_screening',  gb_year_dim], 
                                             indices=[tb_household_indices, tb_screening_indices, gb_year_indices]),
        TB_HHDiagnosticOutputTag : gb_shape('Household screening output', '3D float',
                                             dim=['tb_household', 'tb_diagnosis',  gb_year_dim], 
                                             indices=[tb_household_indices, tb_diagnosis_indices, gb_year_indices]),  
        TB_HHTargetPopsTag       : gb_shape('Household screening output', '3D float',
                                             dim=['tb_household', 'tb_provider_targ_pop',  gb_year_dim], 
                                             indices=[tb_household_indices, tb_provider_targ_pop_indices, gb_year_indices]),
   

        
        # ## Tab ART cohort sizes and TB characteristics
        # ### ART cohort sizes
        TB_ARTCohortSizesTag         : gb_shape('ART Cohort sizes', '3D float', 
                                                dim=['tb_art_all_severity', 'tb_art_age', gb_year_dim],
                                                indices=[tb_art_all_severity, tb_art_age_indices, gb_year_indices]),
        

        # ###  TB characteristics
        TB_ARTCharacteristicsTag     : gb_shape('ART Characteristics', '3D float', 
                                                dim=['tb_art_age', 'tb_art_characteristics', gb_year_dim],
                                                indices=[tb_art_age_indices, tb_art_char_indices, gb_year_indices]),
        
        # ## Tab ART screening
        # ### ART proportion
        TB_ARTProportionTag           : tb_art_year_shape("ART proportions"),

        # ### ART sensitivity amd specificity
        TB_ARTScreenSensitivityTag : tb_art_shape("ART screening sensiticity"),#art_sensitivity, # np.full((TB_ARTAgeLen), 0.90),
        TB_ARTScreenSpecificityTag : tb_art_shape("ART screening specificity"),#art_specificity, # np.full((TB_ARTAgeLen), 0.99),  
        TB_ARTScreenAlgOptionsTag  : gb_shape('ART screening algorithm options','3D str',
                                              dim=['tb_art_severity', 'tb_art_age', 'tb_screening_options'], 
                                              indices = [tb_art_severity_indices, tb_art_age_indices, tb_screening_options],
                                              has_default=False),
        TB_ARTScreenAlgSelectedTag : gb_shape('Selected ART screening algorithm','2D float',
                                                  dim=['tb_art_severity', 'tb_art_age'], 
                                                  indices = [tb_art_severity_indices, tb_art_age_indices],
                                                  options = tb_art_screening_options,
                                                  has_default=False),

        # ### ART screening outputs
        TB_ARTNumScreenedTag           : tb_art_year_shape("ART number of screened"),
        TB_ARTNumReferredTag           : tb_art_year_shape("ART number referred"),
        TB_ARTNumScreenedToReferTag    : tb_art_year_shape("ART number screened to refer"),
        TB_ARTPercTrueCasesTag         : tb_art_year_shape("ART percent of true cases"), 

        # ## Tab ART diagnostic evaluation
        # ###  TB Case type split by treatment and sputum status
        #                : [diagnostic_db['art']['values']]*TB_ARTSevereLen,
        #                   : [diagnostic_db['art']['IDs']]*TB_ARTSevereLen,
        #               : [diagnostic_db['art']['methods']]*TB_ARTSevereLen,
        #           : [diagnostic_db['art']['sensitivity']]*TB_ARTSevereLen,
        #           : [diagnostic_db['art']['specificity']]*TB_ARTSevereLen,

        TB_ARTDiagCaseValuesTag            : gb_shape('ART diagnostic values', '4D float',
                                                  dim=['tb_art_severity', 'tb_art_age',  'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_art_severity_indices, tb_art_age_indices, [], []]),
        TB_ARTDiagCaseIDsTag               : gb_shape('ART diagnostic IDs', '4D int',
                                                  dim=['tb_art_severity', 'tb_art_age',  'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_art_severity_indices, tb_art_age_indices, [], []]),
        TB_ARTDiagCaseMethodsTag         : gb_shape('ART diagnostic Method IDs', '3D int',
                                                  dim=['tb_art_severity', 'tb_art_age', 'tb_dynamic'], 
                                                  indices = [tb_art_severity_indices, tb_art_age_indices, [], []]),
        TB_ARTDiagCaseSensitivityTag       : gb_shape('ART diagnostic Sesnitivity', '4D float',
                                                  dim=['tb_art_severity', 'tb_art_age', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_art_severity_indices, tb_art_age_indices, [], []]),
        TB_ARTDiagCaseSpecificityTag       : gb_shape('ART diagnostic Specificity', '4D float',
                                                  dim=['tb_art_severity', 'tb_art_age', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_art_severity_indices, tb_art_age_indices, [], []]),

        # ### ART sensitivity amd specificity
        TB_ARTDiagSensitivityTag : tb_art_shape("ART diagnostic sensitivity"),
        TB_ARTDiagSpecificityTag : tb_art_shape("ART diagnostic specificity"),

        # ### Diagnosis outputs
        TB_ARTNumDiagnosedPTBTag       : tb_art_year_shape("ART number diagnosed with PTB"),
        TB_ARTNumConfirmedPTBTag       : tb_art_year_shape("ART number confirmed with PTB"),

        # ## Tab ART disease and treatment
        TB_ARTPropLinkedTreatTag       : tb_art_year_shape("ART proportion linked with treatment"),
        TB_ARTNumInitTreatmentTag      : tb_art_year_shape("ART number initialized on treatment"),
        
        # ## Tab ART prevention
        TB_ARTPrvntInActiveEligibleTag  : tb_art_year_shape("ART prevention inactive eligible"),
        TB_ARTPrvntReceivePTPrevTag     : tb_art_year_shape('ART prevention received PTB prevention'),   
        TB_ARTPropPrvntPresumpTag       : tb_art_year_shape("ART proportion prevention presumed"),
        TB_ARTPropPrvntTestedTag        : tb_art_year_shape("ART proportion prevention tested"),
        
        TB_ARTPrvntMethodsTag           : gb_shape('Household prevention method values', '3D float',
                                             dim=['tb_art_severity', 'tb_art_age',  'tb_dynamic_id'], 
                                             indices=[tb_art_severity_indices, tb_art_age_indices, []]),    
        TB_ARTPrvntMethodIDsTag         : gb_shape('Household prevention method IDs', '3D int',
                                             dim=['tb_art_severity', 'tb_art_age',  'tb_dynamic_id'], 
                                             indices=[tb_art_severity_indices, tb_art_age_indices, []],
                                             options=tb_test_ids), 
        TB_ARTPropEligiblePrvntLinkTag  : tb_art_year_shape("ART proportion eligible prevention link"),
        TB_ARTNumberOnTPTTag            : tb_art_year_shape("ART Number initiated on TB preventative treatment"),
        TB_ARTTPTCompleteRateTag        : tb_art_year_shape("ART TB prevention completion rate"),
        TB_ARTNumberCompleteTPTTag      : tb_art_year_shape("ART Number complete preventative treatment"),


        # ## Tab ART output
        TB_ARTScreeningOutputTag  : gb_shape('ART screening output', '4D float',
                                             dim=['tb_art_severity', 'tb_art_age', 'tb_screening',  gb_year_dim], 
                                             indices=[tb_art_severity_indices, tb_art_age_indices, tb_screening_indices, gb_year_indices]),
        TB_ARTDiagnosticOutputTag : gb_shape('ART screening output', '4D float',
                                             dim=['tb_art_severity', 'tb_art_age', 'tb_diagnosis',  gb_year_dim], 
                                             indices=[tb_art_severity_indices, tb_art_age_indices, tb_diagnosis_indices, gb_year_indices]), 
        TB_ARTTargetPopsTag       : gb_shape('ART screening output', '4D float',
                                             dim=['tb_art_severity', 'tb_art_age', 'tb_provider_targ_pop',  gb_year_dim], 
                                             indices=[tb_art_severity_indices, tb_art_age_indices, tb_provider_targ_pop_indices, gb_year_indices]),

        # ## Tab HRG demographics
        TB_HRGDemographicsTag      : gb_shape('HRG Demographic data','1D int',
                                               dim=['tb_highriskgroup', 'tb_highriskgroup_demographics', gb_year_dim], 
                                               indices = [tb_hrg_indices, tb_hrg_demographic_indices, gb_year_indices]),

        # ## Tab HRG screening
        # ### HRG proportion
        TB_HRGProportionTag        : tb_hrg_year_shape('High risk group proportions'), 

        # ### HRG sensitivity amd specificity
        TB_HRGScreenSensitivityTag : tb_hrg_shape('High risk group screening sensitivity'),
        TB_HRGScreenSpecificityTag : tb_hrg_shape('High risk group screening specificity'), 
        TB_HRGScreenAlgOptionsTag  : gb_shape('High risk group screening algorithm options','2D str',
                                               dim=['tb_highriskgroup', 'tb_screening_options'], 
                                              indices = [tb_hrg_indices, tb_screening_options],
                                              has_default=False),
        TB_HRGScreenAlgSelectedTag : gb_shape('Selected high risk group screening algorithm','1D int',
                                               dim=['tb_highriskgroup'], 
                                               indices = [tb_hrg_indices],
                                               options = tb_hrg_screening_options,
                                               has_default=False),

        # ### HRG screening outputs
        TB_HRGNumScreenedTag           : tb_hrg_year_shape('High risk group number screened'),
        TB_HRGNumReferredTag           : tb_hrg_year_shape('High risk group number referred'),
        TB_HRGNumScreenedToReferTag    : tb_hrg_year_shape('High risk group number screned to refer'),
        TB_HRGPercTrueCasesTag         : tb_hrg_year_shape('High risk group percent of true cases'),

        # ## Tab HRG diagnostic evaluation
        # ###  TB Case type split by treatment and sputum status

        TB_HRGDiagCaseValuesTag            : gb_shape('High risk group diagnostic values', '3D float',
                                                  dim=['tb_highriskgroup', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_hrg_indices, [], []]),
        TB_HRGDiagCaseIDsTag               : gb_shape('High risk group diagnostic IDs', '3D int',
                                                  dim=['tb_highriskgroup', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_hrg_indices, [], []]),
        TB_HRGDiagCaseMethodsTag         : gb_shape('High risk group diagnostic Method IDs', '2D int',
                                                  dim=['tb_highriskgroup', 'tb_dynamic'], 
                                                  indices = [tb_hrg_indices, [], []]),
        TB_HRGDiagCaseSensitivityTag       : gb_shape('High risk group diagnostic Sesnitivity', '3D float',
                                                  dim=['tb_highriskgroup', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_hrg_indices, [], []]),
        TB_HRGDiagCaseSpecificityTag       : gb_shape('High risk group diagnostic Specificity', '3D float',
                                                  dim=['tb_highriskgroup', 'tb_dynamic', 'tb_dynamic' ], 
                                                  indices = [tb_hrg_indices, [], []]),

        # ### HRG sensitivity amd specificity
        TB_HRGDiagSensitivityTag : tb_hrg_shape('High risk group diagnostic sensitivity'), # np.full((TB_HRG_Len),  db['Sensitivity']['High risk group'][0]),
        TB_HRGDiagSpecificityTag : tb_hrg_shape('High risk group diagnostic specificity'), # np.full((TB_HRG_Len),  db['Specificity']['High risk group'][0]),  

        # ### Diagnosis outputs
        TB_HRGNumDiagnosedPTBTag       : tb_hrg_year_shape('High risk group number diagnosted with PTB'),
        TB_HRGNumConfirmedPTBTag       : tb_hrg_year_shape('High risk group number confirmed with PTB'),

        # ## Tab HRG disease and treatment
        TB_HRGPropLinkedTreatTag       : tb_hrg_year_shape('High risk group proportion linked with treatment'),
        TB_HRGNumInitTreatmentTag      : tb_hrg_year_shape('High risk group initialized with with treatment'),
        
        # ## Tab HRG prevention
        TB_HRGPrvntInActiveEligibleTag  : tb_hrg_year_shape('High risk group prevention inactive eligible'),
        TB_HRGPrvntReceivePTPrevTag     : tb_hrg_year_shape('High risk group prevention received PTB prevention'),   
        TB_HRGPropPrvntPresumpTag       : tb_hrg_year_shape('High risk group proportion prevention presumed'),
        TB_HRGPropPrvntTestedTag        : tb_hrg_year_shape('High risk group proportion prevention tested'),
        
        TB_HRGPrvntMethodsTag           : gb_shape('Household prevention method values', '2D float',
                                             dim=['tb_highriskgroup', 'tb_dynamic_id'], 
                                             indices=[tb_hrg_indices, []]),    
        TB_HRGPrvntMethodIDsTag         : gb_shape('Household prevention method IDs', '2D int',
                                             dim=['tb_highriskgroup', 'tb_dynamic_id'], 
                                             indices=[tb_hrg_indices, []],
                                             options=tb_test_ids), 
        TB_HRGPropEligiblePrvntLinkTag  : tb_hrg_year_shape('High risk group eligible prevention linked'),
        TB_HRGNumberOnTPTTag            : tb_hrg_year_shape("High risk group Number initiated on TB preventative treatment"),
        TB_HRGTPTCompleteRateTag        : tb_hrg_year_shape("High risk group TB prevention completion rate"),
        TB_HRGNumberCompleteTPTTag      : tb_hrg_year_shape("High risk group Number complete preventative treatment"),
        ## Tab HRG soutput
        TB_HRGScreeningOutputTag       : gb_shape('High risk group screening output', '3D float',
                                             dim=['tb_highriskgroup', 'tb_screening',  gb_year_dim], 
                                             indices=[tb_hrg_indices, tb_screening_indices, gb_year_indices]), 
        TB_HRGDiagnosticOutputTag      : gb_shape('High risk group screening output', '3D float',
                                             dim=['tb_highriskgroup', 'tb_diagnosis',  gb_year_dim], 
                                             indices=[tb_hrg_indices, tb_diagnosis_indices, gb_year_indices]),  
        TB_HRGTargetPopsTag            : gb_shape('High risk group screening output', '3D float',
                                             dim=['tb_highriskgroup', 'tb_provider_targ_pop',  gb_year_dim], 
                                             indices=[tb_hrg_indices, tb_provider_targ_pop_indices, gb_year_indices]),


        TB_ResistProfileTag  : gb_shape('Propotion of prevention', '4D float',
                                         dim=['tb_cases', 'tb_resistance_ages', 'tb_rr_profile', gb_year_dim], 
                                         indices=[tb_case_indices, tb_resistance_age_indices, tb_resistance_profile_indices, gb_year_indices]),
        TB_ResistPropNonSevereTag     : tb_case_resistance_shape('Resistance proportion non-severe'),
        TB_ResistPropLungSurgeryTag   : tb_case_resistance_shape('Resistance proportion lung surgery'),
        TB_ResistMeningitisTag        : tb_case_resistance_shape('Resistance meningitis'),
        TB_ResistPropTreatExhaustTag  : tb_case_resistance_shape('Resistance proportion treated exhaustively'),

        TB_PatientTestNumPTB  : gb_year_shape('Patient test number PTB'),
        TB_PatientTestNumXDR  : gb_year_shape('Patient tet number XDR'),
        TB_PatientTestNumEPTB : gb_year_shape('Patient test number ETB'), 

        TB_CoverageTag : gb_shape('Resistance coverages', '2D float',
                                   dim=['tb_coverages', gb_year_dim], 
                                   indices=[tb_coverage_indices, gb_year_indices]),
        TB_ResistNumToBeTestedTag : gb_shape('Resistance number to be tested', '2D float',
                                              dim=['tb_num_tested', gb_year_dim], 
                                              indices=[tb_num_tested_indices, gb_year_indices]),
        TB_ResistNumTestedTag : gb_shape('Resistance number tested', '2D float',
                                          dim=['tb_num_tested', gb_year_dim], 
                                          indices=[tb_num_tested_indices, gb_year_indices]),
        TB_ResistNumConfirmedTag : gb_shape('Resistance number confirmed', '3D float',
                                             dim=['tb_num_confirmed', 'tb_resistance_ages', gb_year_dim], 
                                             indices=[tb_num_confirmed_indices, tb_resistance_age_indices, gb_year_indices]),
        TB_ResistNumNotConfirmedTag : gb_shape('Resistance number not confirmed', '3D float',
                                                dim=['tb_num_confirmed', 'tb_resistance_ages', gb_year_dim], 
                                                indices=[tb_num_confirmed_indices, tb_resistance_age_indices, gb_year_indices]),


        ## Prevention of TB
        TB_PropPreventionTag : gb_shape('Propotion of prevention', '3D float',
                                          dim=['tb_resistance', 'tb_resistance_ages', gb_year_dim], 
                                          indices=[tb_resistance_indices, tb_resistance_age_indices, gb_year_indices]),


        ## Vaccination
        TB_BCGImmunizationTag : gb_year_shape('BCG immunization'),
        TB_InfantVaccineTag : gb_year_shape('Infant vaccine'),
        TB_AdolAdultVaccineTag : gb_year_shape('Adolescent/Adult vaccine'),

        ## Palliative care
        TB_PalliativeCareTag : gb_shape('Palliative care', '2D float',
                                         dim=['tb_resistance_ages', gb_year_dim], 
                                         indices=[tb_resistance_age_indices, gb_year_indices]), 

        TB_BCGTargetPopTag          : gb_year_shape('BCG target pop'),
        TB_InfantVaccTargPopTag     : gb_year_shape('Infant vaccine target pop'),
        TB_AdultVaccTargPopTag      : gb_year_shape('Adult vaccine target pop'),
        TB_PalliativeCareTargPopTag : gb_shape('Palliative care target population', '2D float',
                                          dim=['tb_resistance_ages', gb_year_dim], 
                                          indices=[tb_resistance_age_indices, gb_year_indices]),



        #Extra target populations
        TB_PreventionTargetTag : gb_shape('Extra prevention target pops', '3D float',
                                          dim=['tb_resistance', 'tb_resistance_ages', gb_year_dim], 
                                          indices=[tb_resistance_indices, tb_resistance_age_indices, gb_year_indices]),

        TB_RR_DST_PTBTag     : gb_year_shape('RR_DST_PTB target pop'),
        TB_RR_DST_ETBTag     : gb_year_shape('RR_DST_ETB target pop'),
        TB_INH_DST_PTBTag    : gb_year_shape('INH_DST_PTB target pop'),
        TB_INH_DST_ETBTag    : gb_year_shape('INH_DST_ETB target pop'),
        TB_BDQ_DST_PTBTag    : gb_year_shape('BDQ_DST_PTB target pop'),
        TB_BDQ_DST_ETBTag    : gb_year_shape('BDQ_DST_ETB target pop'),
        TB_FQ_DST_PTBTag     : gb_year_shape('FQ_DST_PTB target pop'),
        TB_FQ_DST_ETBTag     : gb_year_shape('FQ_DST_ETB target pop'),
        TB_BDQ_FQ_DST_PTBTag : gb_year_shape('BDQ_FQ_DST_PTB target pop'),
        TB_BDQ_FQ_DST_ETBTag : gb_year_shape('BDQ_FQ_DST_ETB target pop'),
        TB_LZD_RR_DST_PTBTag : gb_year_shape('LZD_RR_DST_PTB target pop'),
        TB_LZD_RR_DST_ETBTag : gb_year_shape('LZD_RR_DST_ETB target pop'),
        TB_LZD_FQ_DST_PTBTag : gb_year_shape('LZD_FQ_DST_PTB target pop'),
        TB_LZD_FQ_DST_ETBTag : gb_year_shape('LZD_FQ_DST_ETB target pop'),
        TB_PZA_DSTTag        : gb_year_shape('PZA_DST target pop'),
        TB_ETO_DSTTag        : gb_year_shape('ETO_DST target pop'),
        TB_EMC_DSTTag        : gb_year_shape('EMC_DST target pop'),
        TB_BDQ_LZD_NumDx_U15Tag         : gb_year_shape('BDQ_LZD_NumDx_U15 target pop'),
        TB_BDQ_LZD_NumDx_15PLusTag      : gb_year_shape('BDQ_LZD_NumDx_15PLus target pop'),
        TB_BDQ_LZD_NumDx_PTB_U15Tag     : gb_year_shape('BDQ_LZD_NumDx_PTB_U15 target pop'),
        TB_BDQ_LZD_NumDx_PTB_15PLusTag  : gb_year_shape('BDQ_LZD_NumDx_PTB_15PLus target pop'),
        TB_BDQ_LZD_NumDx_ETB_U15Tag     : gb_year_shape('BDQ_LZD_NumDx_ETB_U15 target pop'),
        TB_BDQ_LZD_NumDx_ETB_15PLusTag  : gb_year_shape('BDQ_LZD_NumDx_ETB_15PLus target pop'),



        TB_AmikacynRifampicinResistant     : tb_pulmonary_year_shape('Amikacyn rifampicin resistant'), 
        TB_EthionamideRifampicinResistant  : tb_pulmonary_year_shape('Ethionamide rifampicin resistant'), 
        TB_PyrazinamideRifampicinResistant : tb_pulmonary_year_shape('Pyrazinamide rifampicin resistant'), 

        TB_Rif_INH_Sen_SevereTag     : tb_child_adult_year_shape('Rif_INH_Sen_Severe'),
        TB_Rif_INH_Sen_NotSevereTag  : tb_child_adult_year_shape('Rif_INH_Sen_NotSevere'),
        TB_Rif_INH_Sen_MeningitisTag : tb_child_adult_year_shape('Rif_INH_Sen_Meningitis'),
        TB_Rif_INH_ResTag            : tb_child_adult_year_shape('Rif_INH_Res'),
        TB_RR_FQ_SenTag              : tb_child_adult_year_shape('RR_FQ_Sen'),
        TB_Rif_FQ_Res_PlusTag        : tb_child_adult_year_shape('Rif_FQ_Res_Plus'),
        TB_BQD_LZD_Tag               : tb_child_adult_year_shape('BQD_LZD'),
        TB_TxFL_Tag                  : tb_child_adult_year_shape('TxFL'),
        TB_TxSL_Tag                  : tb_child_adult_year_shape('TxSL'),
        TB_TxPreXDR_Tag              : tb_child_adult_year_shape('TxPreXDR'),
        TB_LungSurgeryTag            : tb_child_adult_year_shape('Lung surgery'),
        TB_lPTB_ReferredDxTag        : tb_child_adult_year_shape('lPTB_ReferredDx'),

        TB_HIVnClinicallyAssessedTag : tb_pulmonary_child_adult_year_shape('HIV- clinically assessed'), 
        TB_HIVAnyReferredDxTag       : tb_pulmonary_child_adult_year_shape('HIV any referred diagnosed'), 

        # Notification
        TB_HisNotificationTag  : gb_year_shape('Historical notification', has_default=True),
        TB_TargNotificationTag : gb_year_shape('Target notification', has_default=True),
        TB_ResNotificationTag  : tb_impact_year_shape('Result notification', has_default=True),
        TB_CounterfactualNotifTag : gb_year_shape('Counterfactual notification', has_default=True),
        
        

        TB_ResHIVTBCoInfTag    : gb_shape('Result HIV TB Coinfected', '2D float',
                                          dim=['tb_hiv_coinfected', gb_year_dim], 
                                          indices=[tb_hiv_coinfected_indices, gb_year_indices]),


        TB_ResIncidenceTag  : tb_impact_year_shape('Result incidence', has_default=True),
        TB_ResMortalityTag  : tb_impact_year_shape('Result mortality', has_default=True),
        TB_ResPrevalenceTag : tb_impact_year_shape('Result prevalence', has_default=True),
        
        TB_UserEnteredIncidenceTag  : gb_year_shape('User entered incidence', has_default=True),
        TB_UserEnteredMortalityTag  : gb_year_shape('User entered mortality', has_default=True),
        TB_AlternativeProjBurdenTag :  gb_bool_shape('Checkbox use alternative projection of TB burden', 
                                                     note='''When this is true, then the calculations will use 
                                                             TB_UserEnteredIncidenceTag and TB_UserEnteredMortalityTag '''), 

        TB_FortIncidenceTag    : tb_resbounds_year_shape('Fort incidence', has_default=True),
        TB_FortNotificationTag : tb_resbounds_year_shape('Fort notification', has_default=True),
        TB_FortPrevalenceTag   : tb_resbounds_year_shape('Fort prevalence', has_default=True),
        TB_FortMortalityTag    : tb_resbounds_year_shape('Fort mortality', has_default=True),


        TB_ResPrevalencePctTag : gb_year_shape('Result prevalence percent', has_default=True), 

        TB_CounterfactualScenarioTag : gb_shape('Counterfactual scenario', '2D float',
                                          dim=['tb_scenarios', gb_year_dim], 
                                          indices=[tb_scenario_indices, gb_year_indices]),
        TB_CurrentScenarioTag        :  gb_shape('Current scenario', '2D float',
                                          dim=['tb_scenarios', gb_year_dim], 
                                          indices=[tb_scenario_indices, gb_year_indices]),
        TB_Baseline_HRi_allTag : gb_year_shape('Baseline HRi all'),
        TB_Scaleup_HRi_allTag  : gb_year_shape('Scaleup HRi all'),

        TB_NotificationByCaseTypeTag :  gb_shape('Notification by case type', '2D float',
                                          dim=['tb_notification_cases', gb_year_dim], 
                                          indices=[tb_notification_indices, gb_year_indices], has_default=True),

        TB_PercNotificationTag       :  gb_shape('Notification base year by case type percents', '1D float',
                                          dim=['tb_notification_cases'], 
                                          indices=[tb_notification_indices], has_default=True),

        
        TB_FortVsnTag : gb_str_shape('Last fort version used in calculations', has_default=True),
        
        TB_DataSourceTag : gb_str_shape('Data source', has_default=True), 

        TB_FinalHistoricalYearTag : gb_int_shape('Final historical year', has_default=True), 
        TB_FortInputVersionTag : gb_str_shape('Fort input version', has_default=True), 
        TB_FortInputTag : gb_shape('Fort input object', 'dict', has_default=True), 
        TB_FortInputScaleupTag : gb_shape('Fort input scaleup object', 'dict', has_default=True), 
        TB_FortOutputVersionTag : gb_str_shape('Fort output version', has_default=True), 
        TB_FortOutputTag : gb_shape('Fort output object', 'dict', has_default=True), 
        

    }




from AvenirCommon.Util import GBRange
from SpectrumCommon.Const.TB import *
from SpectrumCommon.Const.PJ.PJNTags import PJN_ISO3AlphaTag, PJN_FinalYearTag
from SpectrumCommon.Modvars.TB.TBNotifDefs import apply_notif_split
import SpectrumCommon.External.TB.TBFortWrapper as fort



def delete_modvar(projection, tag):
    if tag in projection:
        del projection[tag]

def TB_PreUpdateModvars(projection, created_projection):
    if TB_DataSourceTag not in projection:
        # Projections created prior to the addition of this modvar should all have the same TB data source (WHO 2023)
        projection[TB_DataSourceTag] = 'WHO 2023'
        
    if TB_FinalHistoricalYearTag not in projection:
        projection[TB_FinalHistoricalYearTag] = TB_FinalHist_Year

    if TB_FortInputVersionTag not in projection:
        if (projection[TB_UseOldFailsafeTag]):
            projection[TB_FortInputVersionTag] = 'V1'
            projection[TB_FortOutputVersionTag] = 'V2'
        elif  projection[TB_UseOldIPTag]:
            projection[TB_FortInputVersionTag] = 'V2'
            projection[TB_FortOutputVersionTag] = 'V3'
        else: 
            projection[TB_FortInputVersionTag] = 'V6'
            projection[TB_FortOutputVersionTag] = 'V6'

        ISO3 = projection[PJN_ISO3AlphaTag]
        finalYear = projection[PJN_FinalYearTag]
        base_year = projection[TB_BaseYearTag]# deafult to TB_FirstHist_Year
        fort_base_index = max(2015, base_year)-TB_FirstHist_Year
        fort_slice = slice(fort_base_index, min(TB_Fort_Final_Year, finalYear) -TB_FirstHist_Year+1)
        tb_slice  = slice(max(2015-base_year, 0), min(TB_Fort_Final_Year, finalYear)-base_year+1)

        projection[TB_FortInputTag] = fort.get_tb_fort_inputs(ISO3, finalYear, tb_slice, fort_slice, 
                                                              None, None, None, 
                                                              fort_vsn=projection[TB_FortInputVersionTag])
        projection[TB_FortOutputTag] = fort.get_tb_fort_outputs(ISO3, projection[TB_FortOutputVersionTag])
        if projection[TB_FortInputVersionTag]  in ['V1', 'V2']:
            projection[TB_FortInputTag]["sEp"] = []
            projection[TB_FortInputTag]["tXf"] = []
            projection[TB_FortInputTag]["pHat"] = []
            for yr in GBRange(projection[TB_FortInputTag]["year"][0], projection[TB_FortInputTag]["year"][-1]):
                projection[TB_FortInputTag]["sEp"].append(None)
                projection[TB_FortInputTag]["pHat"].append(None)
                projection[TB_FortInputTag]["tXf"].append(0.035)

            for yr in GBRange(projection[TB_FortInputTag]["year"][-1]+1, TB_Fort_Final_Year):
                projection[TB_FortInputTag]["year"].append(yr)    
                for key in ["iHat", "sEi", "nHat", "sEn", "mHat", "sEm", "pHat", "sEp"]:
                    projection[TB_FortInputTag][key].append(None)
                projection[TB_FortInputTag]["tXf"].append(0.035)
            projection[TB_FortInputTag]["hRd"] = [1]*len(projection[TB_FortInputTag]["year"]) 
            projection[TB_FortInputTag]["hRi"] = [1]*len(projection[TB_FortInputTag]["year"]) 
            projection[TB_FortInputTag]["oRt"] = [1]*len(projection[TB_FortInputTag]["year"])
        pass

    return projection

def TB_UpdateModvars(projection, created_projection):

    # if TB_OldTag in projection:
    #     projection[TB_NewOldTag] = new_modvars[TB_NewOldTag].copy()
    #     del projection[TB_OldTag]
    if TB_NotificationByCaseTypeTag_V1 in projection:
        notif = projection[TB_TargNotificationTag]
        notif_pct   = created_projection[TB_PercNotificationTag]
        notif_split = created_projection[TB_NotificationByCaseTypeTag]
        notif_split = apply_notif_split(created_projection[TB_BaseYearTag], notif_split, notif, notif_pct)
        projection[TB_NotificationByCaseTypeTag] = notif_split
    # if TB_PercNotificationTag_V1 in projection:
    if TB_BaseYear_V1 in projection:
        projection[TB_UseOldFailsafeTag] = projection[TB_BaseYear_V1] == 2021
        projection[TB_UseOldIPTag] = projection[TB_BaseYear_V1] == 2022
    if TB_HistoricalTag_V1 in projection:
        projection[TB_HistNotifStartYearTag] = projection[TB_HistoricalTag_V1] ['notifications']['startYear']
        for key in TB_NOTIF_DB_KEYS:
            projection[TB_HistNotifTag][key] = projection[TB_HistoricalTag_V1] ['notifications'][key]
        for key in TB_NOTIF_DIST_KEYS:
            projection[TB_HistNotifTag][key] = projection[TB_HistoricalTag_V1] ['Noti_Distribution'][key]
    if TB_HHPrvntMethodsTag_V1 in projection:
            projection[TB_HHPrvntMethodsTag][TB_HH5_14] = projection[TB_HHPrvntMethodsTag_V1][TB_HH5_14] 
            projection[TB_HHPrvntMethodsTag][TB_HH15p] = projection[TB_HHPrvntMethodsTag_V1][TB_HH15p] 
    
    # delete tags
    for tag in TB_TagsForDeletion:
        if tag in projection:
            del projection[tag]

    return projection

TB_ARTDiagCaseValuesTag_V1          = '<TB_ARTDiagCaseValues_V1>'                      
TB_ARTDiagCaseIDsTag_V1             = '<TB_ARTDiagCaseIDs_V1>'                    
TB_ARTDiagCaseMethodsTag_V1         = '<TB_ARTDiagCaseMethodIDs_V1>'   
TB_ARTDiagCaseSensitivityTag_V1     = '<TB_ARTDiagCaseSensitivity_V1>'
TB_ARTDiagCaseSpecificityTag_V1     = '<TB_ARTDiagCaseSpecificity_V1>'  
TB_NotificationByCaseTypeTag_V1     = '<TB_NotificationByCaseType_V1>'
TB_PercNotificationTag_V1           = '<TB_PercentNotification_V1>'      
TB_HHPrvntMethodsTag_V1           = '<TB_HHPrvntMethods_V1>'  
TB_HHPrvntMethodIDsTag_V1         = '<TB_HHPrvntMethodIDs_V1>'   

TB_BaseYear_V1         = '<TB_BaseYear_V1>' 
TB_HistoricalTag_V1    = '<TB_HistoricalData_V1>'

TB_TagsForDeletion = [
    TB_ARTDiagCaseValuesTag_V1, 
    TB_ARTDiagCaseIDsTag_V1,    
    TB_ARTDiagCaseMethodsTag_V1,
    TB_ARTDiagCaseSensitivityTag_V1,
    TB_ARTDiagCaseSpecificityTag_V1,
    TB_NotificationByCaseTypeTag_V1,
    TB_PercNotificationTag_V1,
    TB_BaseYear_V1,
    TB_HistoricalTag_V1,
    TB_HHPrvntMethodsTag_V1,
    TB_HHPrvntMethodIDsTag_V1

]


from os import environ, getcwd
from requests import get
import time

from AvenirCommon.Database import GB_get_db_json


from SpectrumCommon.Const.PJ import PJN_ISO3AlphaTag, PJN_FinalYearTag
import SpectrumCommon.Const.TB as tbc
from SpectrumCommon.Const.DP import DP_BigPopTag
from SpectrumCommon.Const.GB import GB_BothSexes

from SpectrumCommon.Const.GB.GBVersions import TB_FORT_OUTPUT_VERSION, TB_WHO_DB_VERSION
from SpectrumCommon.External.TB.FinalModel2.SpectrumWebConnect import run_dynamical_model
# from Calc.TB.TBEstimateIntervImpacts import TB_EstimateInterventionImpacts

import numpy as np
from loguru import logger


def get_tb_dynamical_outputs(ISO3, reported_year=None, reported_notif=None, vsn=TB_FORT_OUTPUT_VERSION):
    fort_outputs = GB_get_db_json(environ['AVENIR_SW_DEFAULT_DATA_CONNECTION'], 'tuberculosis', f'fort/outputs/{ISO3}_{vsn}.JSON')

    # Rescaling notification to the reported 2021 values
    # idx   = reported_year-TB_FirstHist_Year
    # scale = reported_notif/fort_outputs['nMid'][idx]
    # fort_outputs['nMid'][idx:] = scale*np.array(fort_outputs['nMid'][idx:])
    # fort_outputs['nLo'][idx:]  = scale*np.array(fort_outputs['nLo'][idx:])
    # fort_outputs['nHi'][idx:]  = scale*np.array(fort_outputs['nHi'][idx:])
    # fort_outputs['nSd'][idx:]  = scale*np.array(fort_outputs['nSd'][idx:])
 
    return fort_outputs


def get_tb_dynamical_version():
    fort_url = environ.get("AVENIR_SW_FORT_URL", "https://tbbetafort-server.azurewebsites.net")
    response = get(f'{fort_url}/version')
    return response.json()['version']

def call_tb_dynamical(ISO3, prm_modvar, xs_modvar, pop_data_modvar, Hi_scn, Hi_ref, TxTSFL_scn, TxTSFL_ref, Hd_scn, Hd_ref, firstImpactYr, finalImpactYr, finalYr):
    epi_outputs_base = None
    final_model_path = getcwd()+ '/SpectrumCommon/External/TB/FinalModel2'
    # ones = Hd_scn/Hd_scn
    # Hd_scn=ones*1.3
    # Hd_ref=ones*1


    # Hi_scn=ones*0.03
    # Hi_ref=ones*0.02

    # TxTSFL_scn=ones*0.90
    # TxTSFL_ref=ones*0.84
    
    # with open(f'TB_DYN_MODEL/{ISO3}/inputs/PRM_Modvar.JSON', 'w+') as fp:
    #     ujson.dump(prm_modvar, fp)
    # with open(f'TB_DYN_MODEL/{ISO3}/inputs/xs_Modvar.JSON', 'w+') as fp:
    #     ujson.dump(xs_modvar, fp)
    # with open(f'TB_DYN_MODEL/{ISO3}/inputs/pop_data_Modvar.JSON', 'w+') as fp:
    #     pop_data_modvar['HIV_prev'] = pop_data_modvar['HIV_prev'].tolist()
    #     ujson.dump(pop_data_modvar, fp)
    #     pop_data_modvar['HIV_prev'] = np.array(pop_data_modvar['HIV_prev'])
    # with open(f'TB_DYN_MODEL/{ISO3}/inputs/Hi_scn.JSON', 'w+') as fp:
    #     ujson.dump(Hi_scn, fp)
    # with open(f'TB_DYN_MODEL/{ISO3}/inputs/Hi_ref.JSON', 'w+') as fp:
    #     ujson.dump(Hi_ref, fp)
    # with open(f'TB_DYN_MODEL/{ISO3}/inputs/TxTSFL_scn.JSON', 'w+') as fp:
    #     ujson.dump(TxTSFL_scn, fp)
    # with open(f'TB_DYN_MODEL/{ISO3}/inputs/TxTSFL_ref.JSON', 'w+') as fp:
    #     ujson.dump(TxTSFL_ref, fp)
    # with open(f'TB_DYN_MODEL/{ISO3}/inputs/Hd_ref.JSON', 'w+') as fp:
    #     ujson.dump(Hd_ref, fp)
    # with open(f'TB_DYN_MODEL/{ISO3}/inputs/Hd_scn.JSON', 'w+') as fp:
    #     ujson.dump(Hd_scn, fp)
    # Outputs_Scn = [[np.ones(9)]*8, [np.ones(9)]*8]
    Outputs_Scn = run_dynamical_model(ISO3, prm_modvar, xs_modvar, pop_data_modvar, Hi_scn, Hi_ref, TxTSFL_scn, TxTSFL_ref, Hd_scn, Hd_ref, firstImpactYr, finalImpactYr, finalYr, final_model_path)
    # with open(f'TB_DYN_MODEL/{ISO3}/results/model_outputs_scn.JSON', 'r') as fp: 
    #     Outputs_Scn = ujson.load(fp)
        
    #     for i, scen in enumerate(Outputs_Scn):
    #         for j, ind in enumerate(Outputs_Scn[i]):
    #             Outputs_Scn[i][j] = np.array(Outputs_Scn[i][j])
    # for i,e in enumerate(prm_modvar['rHIV']):
    #     prm_modvar['rHIV'][i] = e.tolist() 
    # prm_modvar['N'] = prm_modvar['N'].tolist() 
    # prm_modvar['gth'] = prm_modvar['gth'].tolist()
    # prm_modvar['r']['ageing'] = prm_modvar['r']['ageing'].tolist()
    # prm_modvar['r']['self_cure'] = prm_modvar['r']['self_cure'].tolist()
    # if 'error' in fort_outputs:
        # raise Exception('TB Fort API called failed')
        
    # with open(f'TB_DYN_MODEL/{ISO3}/results/model_outputs_scn.JSON', 'w+') as fp:
    #     for i, scen in enumerate(Outputs_Scn):
    #         for j, ind in enumerate(Outputs_Scn[i]):
    #             Outputs_Scn[i][j] = Outputs_Scn[i][j].tolist()
    #     ujson.dump(Outputs_Scn, fp)
    #     for i, scen in enumerate(Outputs_Scn):
    #         for j, ind in enumerate(Outputs_Scn[i]):
    #             Outputs_Scn[i][j] = np.array(Outputs_Scn[i][j])
               
    return Outputs_Scn


def calculate_dynamical_impacts(modvars):
    ISO3       = modvars[PJN_ISO3AlphaTag]
    final_year = modvars[PJN_FinalYearTag]
    final_history_year = modvars[tbc.TB_FinalHistoricalYearTag]
    base_year  = modvars[tbc.TB_BaseYearTag]# deafult to TB_FirstHist_Year
    # yr_len = final_year - base_year + 1
    # import ujson
    # with open('TB_INDIA_11_20.JSON', 'w+') as fp:
    #     ujson.dump(modvars, fp)
    fort_out_version = modvars[tbc.TB_FortOutputVersionTag ]
    fort_outputs = GB_get_db_json(environ['AVENIR_SW_DEFAULT_DATA_CONNECTION'], 'tuberculosis', f'fort/outputs/{ISO3}_{fort_out_version}.JSON') 
    start = time.time()
    modvars[tbc.TB_DynPopDataTag]['HIV_prev'] =  np.array(modvars[tbc.TB_DynPopDataTag]['HIV_prev'])
    if len(modvars[tbc.TB_DynPRMTag]['r']['ageing'])==5 :
        modvars[tbc.TB_DynPRMTag]['r']['ageing'] = modvars[tbc.TB_DynPRMTag]['r']['ageing'][1: ]
       
    firstImpactYr  = modvars[tbc.TB_DynamicalFirstYearTag]
    finalImpactYr  = modvars[tbc.TB_DynamicalFinalYearTag]
    fiirstImpactIdx = firstImpactYr - base_year
    finalImpactIdx = finalImpactYr - base_year
    dyn_yr_slice = slice(0, final_year-base_year+1)
    Outputs_Scn = call_tb_dynamical(ISO3, 
                                      modvars[tbc.TB_DynPRMTag],
                                      None, # modvars[tbc.TB_DynXSTag],
                                      modvars[tbc.TB_DynPopDataTag],
                                      Hi_scn = modvars[tbc.TB_Scaleup_HRi_allTag][finalImpactIdx],
                                      Hi_ref = modvars[tbc.TB_Baseline_HRi_allTag][finalImpactIdx],
                                      TxTSFL_scn = modvars[tbc.TB_CurrentScenarioTag][tbc.TB_TreatmentSuccess][finalImpactIdx],
                                      TxTSFL_ref = modvars[tbc.TB_CounterfactualScenarioTag][tbc.TB_TreatmentSuccess][finalImpactIdx],
                                      Hd_scn = modvars[tbc.TB_TargNotificationTag][finalImpactIdx],
                                      Hd_ref = modvars[tbc.TB_CounterfactualNotifTag][finalImpactIdx],
                                      firstImpactYr=firstImpactYr,
                                      finalImpactYr=finalImpactYr,
                                      finalYr=final_year)
    
    if type(modvars[tbc.TB_DynPRMTag]['r']['cs_1997'])!=list:
        modvars[tbc.TB_DynPRMTag]['r']['cs_1997']  =  modvars[tbc.TB_DynPRMTag]['r']['cs_1997'].tolist()
        modvars[tbc.TB_DynPRMTag]['r']['cs_2022']  =  modvars[tbc.TB_DynPRMTag]['r']['cs_2022'].tolist()
        modvars[tbc.TB_DynPRMTag]['r']['cs2_1997'] =  modvars[tbc.TB_DynPRMTag]['r']['cs2_1997'].tolist()
        modvars[tbc.TB_DynPRMTag]['r']['cs2_2022'] =  modvars[tbc.TB_DynPRMTag]['r']['cs2_2022'].tolist()
    if len(modvars[tbc.TB_DynPRMTag]['r']['ageing'])==4 :
        modvars[tbc.TB_DynPRMTag]['r']['ageing'] = [0]+modvars[tbc.TB_DynPRMTag]['r']['ageing']
    
    numpy_type = type(np.array([]))
            
    for key in modvars[tbc.TB_DynPopDataTag]:
        if numpy_type == type(modvars[tbc.TB_DynPopDataTag][key]):
            modvars[tbc.TB_DynPopDataTag][key] = modvars[tbc.TB_DynPopDataTag][key].tolist()
    # tb_dyn_output = []
    # for i in range(0,10):
    #     tb_dyn_output.append(np.zeros(len(modvars[tbc.TB_ResNotificationTag][0])))
    # if 'error' in scaleup_results:
    #     modvars['timings']['TB_Dynamical'] = 'Failed'
    #     return None
    if 'timings' not in  modvars:
        modvars['timings'] = {}
    modvars['timings']['TB_Dynamical'] = time.time()-start
    big_pop_ratio = np.sum(modvars[DP_BigPopTag][GB_BothSexes], axis=0)/1e5
    
    for s in [tbc.TB_Counterfactual, tbc.TB_Scaleup]:
        noti = modvars[tbc.TB_ResNotificationTag][s].copy()
        inci = modvars[tbc.TB_ResIncidenceTag][s].copy()
        prev = modvars[tbc.TB_ResPrevalenceTag][s].copy()
        mort = modvars[tbc.TB_ResMortalityTag][s].copy()
        
        noti[dyn_yr_slice] = Outputs_Scn[s][5]*big_pop_ratio[dyn_yr_slice]
        inci[dyn_yr_slice] = Outputs_Scn[s][0]*big_pop_ratio[dyn_yr_slice]
        prev[dyn_yr_slice] = Outputs_Scn[s][7]*big_pop_ratio[dyn_yr_slice]
        mort[dyn_yr_slice] = (Outputs_Scn[s][3]+Outputs_Scn[s][4])*big_pop_ratio[dyn_yr_slice]
        
        logger.debug(f'BigPopRatio {big_pop_ratio}')
        logger.debug(f'noti {noti}')
        logger.debug(f'inci {inci}')
        logger.debug(f'prev {prev}')
        logger.debug(f'mort {mort}')
        
        idx = max(final_history_year-base_year, 0)
        fort_idx = final_history_year-2000
        noti_scale = fort_outputs['nMid'][fort_idx]/noti[idx]
        inci_scale = fort_outputs['iMid'][fort_idx]/inci[idx]
        mort_scale = fort_outputs['mMid'][fort_idx]/mort[idx]

        # noti_scale = fort_outputs['nMid'][tbc.TB_FinalHist_Year-2000]/noti[tbc.TB_FinalHist_Year-base_year]
        # inci_scale = fort_outputs['iMid'][tbc.TB_FinalHist_Year-2000]/inci[tbc.TB_FinalHist_Year-base_year]
        prev_scale = inci_scale
        # mort_scale = fort_outputs['mMid'][tbc.TB_FinalHist_Year-2000]/mort[tbc.TB_FinalHist_Year-base_year]
        logger.debug(f'noti scale {noti_scale}')
        logger.debug(f'inci scale {inci_scale}')
        logger.debug(f'prev scale {prev_scale}')
        logger.debug(f'mort scale {mort_scale}')
        pass
        modvars[tbc.TB_ResNotificationTag][s] = (noti*noti_scale).tolist()
        modvars[tbc.TB_ResIncidenceTag][s]    = (inci*inci_scale).tolist()
        modvars[tbc.TB_ResPrevalenceTag][s]   = (prev*prev_scale).tolist()
        modvars[tbc.TB_ResMortalityTag][s]    = (mort*mort_scale).tolist()
        
    # Notification TB_Counterfactual should match the locked baseline notification 
    # modvars[tbc.TB_ResNotificationTag][tbc.TB_Counterfactual] = modvars[tbc.TB_HisNotificationTag]
        
    # scaleup_HRd, scaleup_HRi, scaleup_HRi_all = TB_EstimateInterventionImpacts(yr_len, HH_Contacts, ART_Cohorts, HRGs, PatientInitiated, modvars[tbc.TB_ResPrevalenceTag][tbc.TB_Counterfactual], pop_sum)
    # modvars[tbc.TB_Scaleup_HRi_allTag] = scaleup_HRi_all

    modvars[tbc.TB_ResNotificationTag][tbc.TB_Difference] = modvars[tbc.TB_ResNotificationTag][tbc.TB_Scaleup] - modvars[tbc.TB_ResNotificationTag][tbc.TB_Counterfactual] 
    modvars[tbc.TB_ResIncidenceTag][tbc.TB_Difference]    = modvars[tbc.TB_ResIncidenceTag][tbc.TB_Counterfactual]    - modvars[tbc.TB_ResIncidenceTag][tbc.TB_Scaleup]
    modvars[tbc.TB_ResMortalityTag][tbc.TB_Difference]    = modvars[tbc.TB_ResMortalityTag][tbc.TB_Counterfactual]    - modvars[tbc.TB_ResMortalityTag][tbc.TB_Scaleup]
    modvars[tbc.TB_ResPrevalenceTag][tbc.TB_Difference]   = modvars[tbc.TB_ResPrevalenceTag][tbc.TB_Counterfactual]    - modvars[tbc.TB_ResPrevalenceTag][tbc.TB_Scaleup]
        
        
    # with open(f'TB_DYN_MODEL/{ISO3}/results/TB_ResNotificationTag.JSON', 'w+') as fp:
    #     ujson.dump(modvars[tbc.TB_ResNotificationTag].tolist(), fp)
    # with open(f'TB_DYN_MODEL/{ISO3}/results/TB_ResIncidenceTag.JSON', 'w+') as fp:
    #     ujson.dump(modvars[tbc.TB_ResIncidenceTag].tolist(), fp)
    # with open(f'TB_DYN_MODEL/{ISO3}/results/TB_ResMortalityTag.JSON', 'w+') as fp:
    #     ujson.dump(modvars[tbc.TB_ResMortalityTag].tolist(), fp)
    # with open(f'TB_DYN_MODEL/{ISO3}/results/TB_ResPrevalenceTag.JSON', 'w+') as fp:
    #     ujson.dump(modvars[tbc.TB_ResPrevalenceTag].tolist(), fp)
    
    db =  GB_get_db_json(environ['AVENIR_SW_DEFAULT_DATA_CONNECTION'], 'tuberculosis', f'countries\\{ISO3}_{TB_WHO_DB_VERSION}.JSON')
    db_year_index = tbc.TB_FinalHist_Year-db['burden']['startYear']
    e_inc_num    = db['burden']['e_inc_num'][db_year_index]
    e_inc_num_lo = db['burden']['e_inc_num_lo'][db_year_index]
    e_inc_num_hi = db['burden']['e_inc_num_hi'][db_year_index]
    
    e_mort_num    = db['burden']['e_mort_num'][db_year_index]
    e_mort_num_lo = db['burden']['e_mort_num_lo'][db_year_index]
    e_mort_num_hi = db['burden']['e_mort_num_hi'][db_year_index]
    
    modvars[tbc.TB_FortNotificationTag][tbc.TB_ResMidIdx]   = modvars[tbc.TB_ResNotificationTag][tbc.TB_Counterfactual]
    modvars[tbc.TB_FortNotificationTag][tbc.TB_ResLowerIdx] = 0.9*modvars[tbc.TB_ResNotificationTag][tbc.TB_Counterfactual]
    modvars[tbc.TB_FortNotificationTag][tbc.TB_ResUpperIdx] = 1.1*modvars[tbc.TB_ResNotificationTag][tbc.TB_Counterfactual]
    modvars[tbc.TB_FortNotificationTag][tbc.TB_ResSdIdx]    = modvars[tbc.TB_ResNotificationTag][tbc.TB_Counterfactual]

    modvars[tbc.TB_FortIncidenceTag][tbc.TB_ResMidIdx]   = modvars[tbc.TB_ResIncidenceTag][tbc.TB_Counterfactual]
    modvars[tbc.TB_FortIncidenceTag][tbc.TB_ResLowerIdx] = modvars[tbc.TB_ResIncidenceTag][tbc.TB_Counterfactual]*(e_inc_num_lo/e_inc_num)
    modvars[tbc.TB_FortIncidenceTag][tbc.TB_ResUpperIdx] = modvars[tbc.TB_ResIncidenceTag][tbc.TB_Counterfactual]*(e_inc_num_hi/e_inc_num)
    modvars[tbc.TB_FortIncidenceTag][tbc.TB_ResSdIdx]    = modvars[tbc.TB_ResIncidenceTag][tbc.TB_Counterfactual]

    modvars[tbc.TB_FortMortalityTag][tbc.TB_ResMidIdx]   = modvars[tbc.TB_ResMortalityTag][tbc.TB_Counterfactual]
    modvars[tbc.TB_FortMortalityTag][tbc.TB_ResLowerIdx] = modvars[tbc.TB_ResMortalityTag][tbc.TB_Counterfactual]*(e_mort_num_lo/e_mort_num)
    modvars[tbc.TB_FortMortalityTag][tbc.TB_ResUpperIdx] = modvars[tbc.TB_ResMortalityTag][tbc.TB_Counterfactual]*(e_mort_num_hi/e_mort_num)
    modvars[tbc.TB_FortMortalityTag][tbc.TB_ResSdIdx]    = modvars[tbc.TB_ResMortalityTag][tbc.TB_Counterfactual]

    modvars[tbc.TB_FortPrevalenceTag][tbc.TB_ResMidIdx]   = modvars[tbc.TB_ResPrevalenceTag][tbc.TB_Counterfactual]
    modvars[tbc.TB_FortPrevalenceTag][tbc.TB_ResLowerIdx] = modvars[tbc.TB_ResPrevalenceTag][tbc.TB_Counterfactual]*(e_inc_num_lo/e_inc_num)
    modvars[tbc.TB_FortPrevalenceTag][tbc.TB_ResUpperIdx] = modvars[tbc.TB_ResPrevalenceTag][tbc.TB_Counterfactual]*(e_inc_num_hi/e_inc_num)
    modvars[tbc.TB_FortPrevalenceTag][tbc.TB_ResSdIdx]    = modvars[tbc.TB_ResPrevalenceTag][tbc.TB_Counterfactual]
    
    
    logger.debug(f'noti after scaling {modvars[tbc.TB_ResNotificationTag][0] }')
        
    pass


        
    #scaleup_prev = calc_prevalence_from_notif_inci(np.array(scaleup_results['nMid']), np.array(scaleup_results['iMid']))[fort_slice]
    # scaleup_prev = scaleup_results['pMid'][fort_slice]
    # modvars[tbc.TB_ResPrevalenceTag][tbc.TB_Scaleup][tb_slice] = np.array(scaleup_prev)


    # # send results to modvars
    # modvars[tbc.TB_CurrentScenarioTag][tbc.TB_AvgProbDetect] = scaleup_HRd
    # modvars[tbc.TB_CurrentScenarioTag][tbc.TB_AvgProbReceivPrvnt] = scaleup_HRi
    # modvars[tbc.TB_Scaleup_HRi_allTag] = scaleup_HRi_all


    # # db =  GB_get_db_json(environ['AVENIR_SW_DEFAULT_DATA_CONNECTION'], 'tuberculosis', 'countries\\'+ISO3+'_V1.JSON')

    # # idx = TB_FinalHist_Year-base_year
    # # db_idx = TB_FinalHist_Year-db['projections']['startYear']
    # # scale = modvars[tbc.TB_HisNotificationTag][idx]/db['projections']['notification'][db_idx]
    # if modvars[tbc.TB_UseOldFailsafeTag]:
    #     fort_vsn = '_V2'
    # elif  modvars[tbc.TB_UseOldIPTag]:
    #     fort_vsn = '_V3'
    # else: 
    #     fort_vsn = '_V5' 
    # fort_outputs = tb_dyn.get_tb_dynamical_outputs(ISO3=ISO3, vsn=fort_vsn)
    # # fort_outputs = GB_get_db_json(environ['AVENIR_SW_DEFAULT_DATA_CONNECTION'], 'tuberculosis', 'fort/outputs/'+ISO3+fort_vsn+'.JSON') 
    
    # if modvars[tbc.TB_UseOldFailsafeTag]:
    #     scale = modvars[tbc.TB_HisNotificationTag][0]/fort_outputs['nMid'][base_year-2000]
    # if modvars[tbc.TB_UseOldIPTag]:
    #     scale = modvars[tbc.TB_HisNotificationTag][0]/fort_outputs['nMid'][base_year-2000]
    # else:
    #     scale = modvars[tbc.TB_HisNotificationTag][tbc.TB_FinalHist_Year-base_year]/fort_outputs['nMid'][tbc.TB_FinalHist_Year-2000]
    
    # # fort_outputs = get_tb_fort_outputs(ISO3, TB_FinalHist_Year, modvars[tbc.TB_HisNotificationTag][0])
    # # if modvars[tbc.TB_UseOldFailsafeTag]:
    #     # scale = modvars[tbc.TB_HisNotificationTag][0]/fort_outputs['nMid'][tbc.TB_FinalHist_Year-2000]

    # modvars[tbc.TB_ResNotificationTag][tbc.TB_Scaleup][tb_slice] = np.array(scaleup_results['nMid'][fort_slice])*scale
    # modvars[tbc.TB_ResIncidenceTag][tbc.TB_Scaleup][tb_slice]    = scaleup_results['iMid'][fort_slice]
    # modvars[tbc.TB_ResMortalityTag][tbc.TB_Scaleup][tb_slice]    = scaleup_results['mMid'][fort_slice]

    
    modvars[tbc.TB_ResNotificationTag][tbc.TB_Difference] = modvars[tbc.TB_ResNotificationTag][tbc.TB_Scaleup] - modvars[tbc.TB_ResNotificationTag][tbc.TB_Counterfactual] 
    modvars[tbc.TB_ResIncidenceTag][tbc.TB_Difference]    = modvars[tbc.TB_ResIncidenceTag][tbc.TB_Counterfactual]    - modvars[tbc.TB_ResIncidenceTag][tbc.TB_Scaleup]
    modvars[tbc.TB_ResMortalityTag][tbc.TB_Difference]    = modvars[tbc.TB_ResMortalityTag][tbc.TB_Counterfactual]    - modvars[tbc.TB_ResMortalityTag][tbc.TB_Scaleup]
    modvars[tbc.TB_ResPrevalenceTag][tbc.TB_Difference]   = modvars[tbc.TB_ResPrevalenceTag][tbc.TB_Counterfactual]    - modvars[tbc.TB_ResPrevalenceTag][tbc.TB_Scaleup]


    # #Projection model results

    # modvars[tbc.TB_FortNotificationTag][tbc.TB_ResMidIdx][tb_slice]   = np.array(scaleup_results['nMid'][fort_slice])*scale
    # modvars[tbc.TB_FortNotificationTag][tbc.TB_ResLowerIdx][tb_slice] = np.array(scaleup_results['nLo'][fort_slice])*scale
    # modvars[tbc.TB_FortNotificationTag][tbc.TB_ResUpperIdx][tb_slice] = np.array(scaleup_results['nHi'][fort_slice])*scale
    # modvars[tbc.TB_FortNotificationTag][tbc.TB_ResSdIdx][tb_slice]    = np.array(scaleup_results['nSd'][fort_slice])*scale

    # modvars[tbc.TB_FortIncidenceTag][tbc.TB_ResMidIdx][tb_slice]   = scaleup_results['iMid'][fort_slice]
    # modvars[tbc.TB_FortIncidenceTag][tbc.TB_ResLowerIdx][tb_slice] = scaleup_results['iLo'][fort_slice]
    # modvars[tbc.TB_FortIncidenceTag][tbc.TB_ResUpperIdx][tb_slice] = scaleup_results['iHi'][fort_slice]
    # modvars[tbc.TB_FortIncidenceTag][tbc.TB_ResSdIdx][tb_slice]    = scaleup_results['iSd'][fort_slice]

    # modvars[tbc.TB_FortMortalityTag][tbc.TB_ResMidIdx][tb_slice]   = scaleup_results['mMid'][fort_slice]
    # modvars[tbc.TB_FortMortalityTag][tbc.TB_ResLowerIdx][tb_slice] = scaleup_results['mLo'][fort_slice]
    # modvars[tbc.TB_FortMortalityTag][tbc.TB_ResUpperIdx][tb_slice] = scaleup_results['mHi'][fort_slice]
    # modvars[tbc.TB_FortMortalityTag][tbc.TB_ResSdIdx][tb_slice]    = scaleup_results['mSd'][fort_slice]

    # modvars[tbc.TB_FortPrevalenceTag][tbc.TB_ResMidIdx][tb_slice]   = scaleup_results['pMid'][fort_slice]
    # modvars[tbc.TB_FortPrevalenceTag][tbc.TB_ResLowerIdx][tb_slice] = scaleup_results['pLo'][fort_slice]
    # modvars[tbc.TB_FortPrevalenceTag][tbc.TB_ResUpperIdx][tb_slice] = scaleup_results['pHi'][fort_slice]
    # modvars[tbc.TB_FortPrevalenceTag][tbc.TB_ResSdIdx][tb_slice]    = scaleup_results['pSd'][fort_slice]
    

    pass



import ujson
import psutil

from fastapi import Request, status
from fastapi.responses import PlainTextResponse, JSONResponse
from fastapi.middleware.cors import CORSMiddleware
import traceback

from AvenirCommon.Util import GB_traceback
from AvenirCommon.Authenticate import authorize_api_to_api, authentication_not_needed

from SpectrumCommon.Const.GB import *
from .ExceptionUtils import SpectrumException
from AvenirCommon.Logger import logger


def setup_middleware(app):
    app.add_middleware(
        CORSMiddleware,
        allow_origins=['*'],
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )
    
    @app.middleware("http")
    async def authorize(request: Request, call_next):
        if (authorize_api_to_api(request, "AVENIR_SW_API_KEY")
        or authentication_not_needed(request)):
            return await call_next(request)
        else:
            return PlainTextResponse("UNAUTHORIZED", status_code=status.HTTP_401_UNAUTHORIZED)
        
        
def setup_exception_handler(app):
    @app.exception_handler(Exception)
    async def debug_exception_handler(request: Request, exc: Exception):
        msg = GB_traceback(exc)
        call_stack = traceback.format_exception(exc)

        errorLog = {
            "subsystem": "Inner System", 
            'cpuPercent': psutil.cpu_percent(), 
            'percentMemoryAvailable': 100 * (psutil.virtual_memory().available / psutil.virtual_memory().total),
            "call_stack": call_stack,
        }
            
        logger.error(ujson.dumps(errorLog))

        response = None

        if hasattr(exc, 'SWErrorCode'):
            content = exc.__dict__
            response = JSONResponse(content=content, status_code=exc.status_code)
        elif hasattr(exc, 'detail'):
            content = { "detail" : exc.detail, "message" : "Old error handler code", "SWErrorCode" : GB_SW_OLDERROR }
            response = JSONResponse(content=content, status_code=500)
        else:
            exc = SpectrumException(GB_SW_UNKNOWN, extra_info=exc.__dict__)
            if 'error' in msg:
                if type(msg['error']) == dict:
                    exc.extra_info = msg['error']
                else:
                    exc.extra_info = {'msg': msg['error']}

            content = exc.__dict__
            response = JSONResponse(content=content, status_code=exc.status_code)
            
        return response
        # return JSONResponse(content={"detail": "Uncaught exception status 500", "call_stack": call_stack }, status_code=500)
    
    
# @app.exception_handler(Exception)
# def debug_exception_handler(request: Request, exc: Exception):    
#     # msg = GB_traceback(exc)
#     # return PlainTextResponse(content=ujson.dumps(msg), media_type="application/json", status_code=msg['status'])

#     return PlainTextResponse(
#         "".join(traceback.format_exception(None, exc, exc.__traceback__)),
#         status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
    # )



from os import environ

from SpectrumCommon.Const.GB import *
from SpectrumCommon.Const.AM import *
from SpectrumCommon.Util.AM.AMUtil import GetDefaultRegionCode
# from ...DP.DATA.DPModvarDef import *
from .AMModvarDef import *
from .AMUtil import getDefaultEpidemicConst
from AvenirCommon.Util import formatCountryFName, GBGetKeyValue
from AvenirCommon.Database import GB_get_db_json, GB_file_exists
from ..GB.GBDefs import *
from .AMPreventionNeeds import *
# AM modvar creates


def init_modvars(countryID, first_year, final_year, extra):
    subnatCode = GBGetKeyValue(extra, 'subNatCode', 0)
        
    connection = environ[GB_SPECT_MOD_DATA_CONN_ENV]
    
    easyAIMCountryName = formatCountryFName(countryID, AMEasyAIMDatabaseVersion)#, subnatCode)
    
    if GB_file_exists(connection, 'aim', 'easyAIM/' + easyAIMCountryName):
        easyAIMData = GB_get_db_json(connection, 'aim', 'easyAIM/' + easyAIMCountryName) 
    else:
        easyAIMData = {}
        
    countryName = formatCountryFName(countryID, AMDatabaseVersion)#, subnatCode)
    if GB_file_exists(connection, 'aim', 'country/' + countryName):
        AMCountryData = GB_get_db_json(connection, 'aim', 'country/' +countryName) 
    else:
        AMCountryData = {}

    
    subnatName = formatCountryFName(countryID, AMDatabaseVersion, subnatCode)
    if GB_file_exists(connection, 'aim', 'country/' + subnatName):
        AMSubnatData = GB_get_db_json(connection, 'aim', 'country/' +subnatName) 
    else:
        AMSubnatData = {}

    initialConditionsCountryName = formatCountryFName(countryID, DPInitialConditionsDBVersion)#, subnatCode)
    if GB_file_exists(connection, 'demproj', DPInitialConditionsDBSubDir + initialConditionsCountryName):
        InitialConditionsDB = GB_get_db_json(connection, 'demproj', DPInitialConditionsDBSubDir + initialConditionsCountryName) 
    else:
        InitialConditionsDB = {}

    AMGlobalData = GB_get_db_json(connection, 'aim', formatCountryFName('AM_Global', AMDatabaseVersion)) 

    region = GetDefaultRegionCode(countryID)    
    
    CSAVRCountryName = formatCountryFName(countryID, AMCSAVRDatabaseVersion)
    CSAVRGlobalName = formatCountryFName('Global', AMCSAVRDatabaseVersion)
    # country-level data
    if GB_file_exists(connection, 'aim', 'CSAVR/' + CSAVRCountryName):                                      
        CSAVRData = GB_get_db_json(connection, 'aim', 'CSAVR/' + CSAVRCountryName) 
    # if no country data, get global
    elif GB_file_exists(connection, 'aim', 'CSAVR/' + CSAVRGlobalName):
        CSAVRData = GB_get_db_json(connection, 'aim', 'CSAVR/' + CSAVRGlobalName)
    # if global is somehow missing, return empty dict
    else:
        CSAVRData = {}
        
    prevNeededName = f'PreventionNeedsDB/{countryID}_{AMPreventionsNeedsVersion}.JSON'
    if GB_file_exists(connection, 'aim', prevNeededName):        
        PreventionNeedsDB = GB_get_db_json(connection, 'aim', prevNeededName)   
    else:
        PreventionNeedsDB = {}

    projInfo = {
        'firstYr'           : first_year,
        'finalYr'           : final_year,
        'firstYrIdx'        : 0,
        'finalYrIdx'        : final_year - first_year,
        'subnatCode'        : GBGetKeyValue(extra, 'subNatCode', 0),
        'region'            : GBGetKeyValue(extra, 'region', region),
        'defaultEpidemic'   : getDefaultEpidemicConst(AMCountryData),
        'epidemicStartYr'   : 1980,
    }
    if 'EpidemicStartYear' in AMCountryData:
        projInfo['epidemicStartYr'] = AMCountryData['EpidemicStartYear']['Start year']

    # let the caller override epidemic start year
    if 'epidemicStartYr' in extra:
        projInfo['epidemicStartYr'] = extra['epidemicStartYr']

    incidenceByFit = IncidenceByFit_Init(projInfo, easyAIMData)
    ARVRegimen = ARVRegimen_Init(projInfo, easyAIMData)

    MVs = {
        AM_InitialConditionsTag : AM_InitialConditions_Init(projInfo, InitialConditionsDB),

        AM_DataSourceTag : '' if easyAIMData == {} else extra['countryDetails']['sources']['AMSource'],
        AM_AMSourcesTag : AMSources_Init(countryID),
        AM_ChangesLogTag : [],
        
        AM_CD4ThreshHoldTag : CD4ThreshHold_Init(projInfo),
        AM_PopsEligTreatTag : PopsEligTreat_Init(easyAIMData),
        AM_AgeHIVChildOnTreatmentTag : AgeHIVChildOnTreatment_Init(projInfo),
        AM_CD4ThreshHoldAdultsTag : CD4ThreshHoldAdults_Init(projInfo),
        AM_InfantFeedingOptionsTag : InfantFeedingOptions_Init(projInfo, easyAIMData, AMGlobalData),
        AM_InfantFeedingOptions_MetaTag : InfantFeedingOptions_Meta_Init(projInfo, AMCountryData, AMGlobalData),
        AM_ARVRegimenTag : ARVRegimen.tolist(),
        AM_ARVRegimen_MetaTag:  {'default': ARVRegimen.tolist()},
        AM_PatientsReallocatedTag : getList_year(projInfo['finalYrIdx']),
        AM_PercentARTDeliveryTag : PercentARTDelivery_Init(projInfo),
        AM_PregTermAbortionPerNumTag : getList_year(projInfo['finalYrIdx']),
        AM_PregTermAbortionTag : getList_year(projInfo['finalYrIdx']),
        AM_MedCD4CountInitTag : getList_year(projInfo['finalYrIdx']),
        AM_PercInterruptedTag : PercLostFollowup_Init(projInfo, easyAIMData), 
        AM_NumberInitTreatmentReinitsTag : getList_year(projInfo['finalYrIdx']),
        AM_NumNewlyInitARTTag : NumNewlyInitART_Init(projInfo),
        AM_PercInterruptedChildTag : getList_year(projInfo['finalYrIdx']),
        AM_NumberInitTreatmentReinitsChildTag : getList_year(projInfo['finalYrIdx']),
        AM_NumNewlyInitARTChildTag : getList_year(projInfo['finalYrIdx']),
        AM_HAARTBySexTag : HAARTBySex_Init(projInfo, easyAIMData),
        AM_CD4CoverageTag : CD4Coverage_Init(projInfo),    
        AM_ARTByAgeInputTypeTag : ARTByAge5YearAG,
        AM_ARTByAge5YearAGTag : getList_sex_5yrAge_year(projInfo['finalYrIdx']),
        AM_ARTByAgeGAMAGTag : ARTByAgeGAMAG_Init(projInfo),
        AM_CovPopsEligTreatTag : CovPopsEligTreat_Init(projInfo),
        AM_ChildTreatInputsTag : ChildTreatInputs_Init(projInfo, easyAIMData),
        AM_ChildARTByAgeGroupPerNumTag : ChildARTByAgeGroupPerNum_Init(projInfo, easyAIMData),
        AM_ANCTestingValuesTag : ANCTestingValues_Init(projInfo),
        AM_NewInfectionsLessImmigrantsTag : getList_sex_singleAge_year(projInfo['finalYrIdx']),
        AM_HIVTestingTag : HIVTesting_Init(projInfo),
        AM_HIVTestingHTSTestsByAgeTag : np.full((GB_Female + 1, DP_GAMAG_A50Plus + 1, projInfo['finalYrIdx'] + 1), DPNotAvail).tolist(),
        AM_Shiny90SurveyDataTag : [],
        AM_Shiny90SurveyData_MetaTag : Shiny90SurveyData_Meta_Init(),
        AM_Shiny90ProgramDataTag : [],
        AM_Shiny90ProgramData_MetaTag : Shiny90ProgramData_Meta_Init(),
        AM_Shiny90AIDSDeathsFYearTag : 0,
        AM_Shiny90PopTag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_Shiny90NumTestsTag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_Shiny90NumTested12MTag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_Shiny90TotalTestsTag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_Shiny90EverTestedTag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_Shiny90DiagnosedTag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_Shiny90NumAwareTag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_Shiny90NumDiagnosesTag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_Shiny90NumDiagnosesModTag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_Shiny90NumLateDiagnosesTag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_Shiny90NotDiagHIV1YrTag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_Shiny90NumFirstTestHIVNeg_Tag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),            
        AM_Shiny90NumReTestHIVNeg_Tag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),               
        AM_Shiny90NumRetestPLHIVOnART_Tag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),           
        AM_Shiny90NumRetestPLHIVNoART_Tag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),           
        AM_Shiny90NumNewDiagnoses_Tag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),               
        AM_Shiny90PosRate_Tag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),                       
        AM_Shiny90YieldNewDiagn_Tag : np.zeros((DP_D_HIVPos + 1, GB_Female + 1, DP_S90_15Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_Shiny90IsFittedTag : False,

        AM_KnowledgeOfStatusInputTypeTag : DP_KoS_CaseReps,
        AM_KnowledgeOfStatusInputTag : np.zeros((DP_Cas_AG_AdultFemale + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_KOSNewDiagnosesChildren0t14Tag : getList_year(projInfo['finalYrIdx'], DPNotAvail),
        AM_KOSNewDiagnosesAdults15PlusTag : getList_year(projInfo['finalYrIdx'], DPNotAvail),
        AM_KOSNewDiagnosesRecCD4TestTag : getList_year(projInfo['finalYrIdx'], DPNotAvail),
        AM_ViralSuppressionInputTypeTag : 0,
        AM_ViralSuppressionInputTag : np.full((DP_Cas_AG_AdultFemale + 1, DP_VS_PLHIVSuppressed + 1, projInfo['finalYrIdx'] + 1), DPNotAvail).tolist(),
        AM_ViralSuppressionThresholdTag : np.full(projInfo['finalYrIdx'] + 1, 1000).tolist(),
        AM_ChildAnnRateProgressLowerCD4Tag : ChildAnnRateProgressLowerCD4_Init(AMGlobalData),
        AM_ChildDistNewInfectionsCD4Tag : ChildDistNewInfectionsCD4_Init(),
        AM_ChildMortByCD4NoARTTag : ChildMortByCD4NoART_Init(),
        AM_ChildMortalityRatesTag : ChildMortalityRates_Init(projInfo, AMGlobalData),
        AM_ChildMortalityRates_MetaTag : {'default': ChildMortalityRates_Meta_Init(projInfo, AMGlobalData)},
        AM_ChildMortalityRatesMultiplierTag : 1,
        AM_ChildMortByCD4WithART0to6Tag : ChildMortByCD4WithART0to6_Init(projInfo, AMGlobalData['ChildMortByCD4WithART0to6Count']), 
        AM_ChildMortByCD4WithART0to6_MetaTag :  {'default' : AMGlobalData['ChildMortByCD4WithART0to6Count']},
        AM_ChildMortByCD4WithART0to6PercTag : ChildMortByCD4WithART0to6Perc_Init(projInfo, AMGlobalData['ChildMortByCD4WithART0to6Perc']),
        AM_ChildMortByCD4WithART0to6Perc_MetaTag :  {'default': AMGlobalData['ChildMortByCD4WithART0to6Perc']},
        AM_ChildMortByCD4WithART7to12Tag : ChildMortByCD4WithART7to12_Init(projInfo, AMGlobalData['ChildMortByCD4WithART7to12Count']),
        AM_ChildMortByCD4WithART7to12_MetaTag :  {'default': AMGlobalData['ChildMortByCD4WithART7to12Count']},
        AM_ChildMortByCD4WithART7to12PercTag : ChildMortByCD4WithART7to12Perc_Init(projInfo, AMGlobalData['ChildMortByCD4WithART7to12Perc']),
        AM_ChildMortByCD4WithART7to12Perc_MetaTag :  {'default': AMGlobalData['ChildMortByCD4WithART7to12Perc']},
        AM_ChildMortByCD4WithARTGT12Tag : ChildMortByCD4WithARTGT12_Init(projInfo, AMGlobalData['ChildMortByCD4WithARTGT12Count']),
        AM_ChildMortByCD4WithARTGT12_MetaTag :  {'default': AMGlobalData['ChildMortByCD4WithARTGT12Count']},
        AM_ChildMortByCD4WithARTGT12PercTag : ChildMortByCD4WithARTGT12Perc_Init(projInfo, AMGlobalData['ChildMortByCD4WithARTGT12Perc']),
        AM_ChildMortByCD4WithARTGT12Perc_MetaTag :  {'default': AMGlobalData['ChildMortByCD4WithARTGT12Perc']},
        AM_ChildARTDistTag : ChildARTDist_Init(projInfo, AMGlobalData),
        AM_ChildARTDist_MetaTag :  {'default': ChildARTDist_Meta_Init(projInfo, AMGlobalData)},
        AM_EffectTreatChildTag : EffectTreatChild_Init(),
        AM_ChildWeightBandsTag : ChildWeightBands_Init(AMGlobalData),
        AM_AdultAnnRateProgressLowerCD4Tag : AdultAnnRateProgressLowerCD4_Init(projInfo, AMGlobalData),
        AM_AdultAnnRateProgressLowerCD4_MetaTag : {'default': AdultAnnRateProgressLowerCD4_Meta_Init(projInfo, AMGlobalData)},
        AM_AdultDistNewInfectionsCD4Tag : AdultDistNewInfectionsCD4_Init(projInfo, AMGlobalData['InitialCD4CountByRegion']),
        AM_AdultDistNewInfectionsCD4_MetaTag :  {'default': AMGlobalData['InitialCD4CountByRegion']},
        AM_AdultMortByCD4NoARTTag : AdultMortByCD4NoART_Init(projInfo, AMGlobalData['MortalityNoARTByRegion']),
        AM_AdultMortByCD4NoART_MetaTag :  {'default': AMGlobalData['MortalityNoARTByRegion']},
        AM_AdultInfectReducTag : AdultInfectReduc_Init(),
        AM_MortalityRatesTag : MortalityRates_Init(projInfo, AMGlobalData),
        AM_MortalityRates_MetaTag :  {'default': MortalityRates_Meta_Init(projInfo, AMGlobalData)},
        AM_MortalityRatesMultiplierTag : 4 if projInfo['region'] == DP_Asia else 1,
        AM_AdultMortByCD4WithART0to6Tag : AdultMortByCD4WithART0to6_Init(projInfo, AMGlobalData['AdultMortByCD4WithART0to6']),
        AM_AdultMortByCD4WithART0to6_MetaTag :  {'default': AMGlobalData['AdultMortByCD4WithART0to6']},
        AM_AdultMortByCD4WithART7to12Tag : AdultMortByCD4WithART7to12_Init(projInfo, AMGlobalData['AdultMortByCD4WithART7to12']),
        AM_AdultMortByCD4WithART7to12_MetaTag :  {'default': AMGlobalData['AdultMortByCD4WithART7to12']},
        AM_AdultMortByCD4WithARTGT12Tag : AdultMortByCD4WithARTGT12_Init(projInfo, AMGlobalData['AdultMortByCD4WithARTGT12']),
        AM_AdultMortByCD4WithARTGT12_MetaTag :  {'default': AMGlobalData['AdultMortByCD4WithARTGT12']}, 
        AM_AdultNonAIDSExcessMortTag : AdultNonAIDSExcessMort_Init(projInfo, AMGlobalData['NonAIDSExcessMortality']),
        AM_AdultNonAIDSExcessMort_Meta_Tag :  {'default': AdultNonAIDSExcessMort_Meta_Init(projInfo, AMGlobalData['NonAIDSExcessMortality'])},
        AM_AdultNonAIDSExcessMortRegionTag : projInfo['region'],
        AM_AdultNonAIDSExcessMortCustomFlagTag : False,
        AM_NonAIDSExcessDeathsSingleAgeTag : np.zeros((DP_OnART+1, GB_Female+1, 80 + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_TFRRegionTag : projInfo['region'],
        AM_TFRRegion_MetaTag : {'default' : projInfo['region']},
        AM_HIVTFRCustomFlagTag : False,  
        AM_HIVTFRTag : HIVTFR_Init(projInfo, AMCountryData, AMGlobalData),
        AM_HIVTFR_MetaTag :  {'default': HIVTFR_Meta_Init(projInfo, AMCountryData, AMGlobalData)},
        AM_TFRInputTypeTag : DP_FRR_SingleYearInput,
        AM_FertCD4DiscountTag : FertCD4Discount_Init(projInfo, AMGlobalData),
        AM_FertCD4Discount_MetaTag :  {'default': FertCD4Discount_Meta_Init(AMGlobalData)},
        AM_RatioWomenOnARTTag : RatioWomenOnART_Init(projInfo, AMGlobalData),
        AM_RatioWomenOnART_MetaTag :  {'default': RatioWomenOnART_Meta_Init(projInfo, AMGlobalData)},
        AM_FRRFitInputTag : FRRFitInput_Init(projInfo),
        AM_FRRbyLocationTag : FRRbyLocation_Init(easyAIMData, AMCountryData),
        AM_TransEffAssumpTag : TransEffAssump_Init(),
        AM_DALYDisabilityWeightsTag : DALYDisabilityWeights_Init(),
        AM_RiskPopOrphansTag : RiskPopOrphans_Init(),
        AM_ECDCValuesTag : ECDCValues_Init(projInfo),
        AM_NosocomialInfectionsByAgeTag : NosocomialInfectionsByAge_Init(projInfo),
        AM_HIVMigrantsBySexAgeTag : getList_sex_5yrAge_year(projInfo['finalYrIdx']),
        AM_HIVMigrantsBySingleAgeTag : getList_sex_singleAge_year(projInfo['finalYrIdx']),

        AM_IncidenceInput1970Tag : IncidenceInput1970_Init(projInfo, easyAIMData),
        AM_CSAVRInputAIDSDeathsTag : CSAVRInputAIDSDeaths_Init(projInfo),
        AM_CSAVRInputAIDSDeathsBySexTag : CSAVRInputAIDSDeathsBySex_Init(projInfo),
        AM_CSAVRInputAIDSDeathsBySexAgeTag : CSAVRInputAIDSDeathsBySexAge_Init(projInfo),
        AM_CSAVRInputAIDSDeathsSourceTag : CSAVRSource1,
        AM_CSAVRInputAIDSDeathsSourceNameTag : [('Source ' + str(i + 1)) for i in GBRange(CSAVRSource1, CSAVRSource3)],
        AM_CSAVRInputNewDiagnosesTag : CSAVRInputNewDiagnoses_Init(projInfo),
        AM_CSAVRInputNewDiagnosesBySexTag : CSAVRInputNewDiagnosesBySex_Init(projInfo),
        AM_CSAVRInputNewDiagnosesBySexAgeTag : CSAVRInputNewDiagnosesBySexAge_Init(projInfo),
        AM_CSAVRInputNewDiagnosesBySexAgeCD4Tag : CSAVRInputNewDiagnosesBySexAgeCD4_Init(projInfo),
        AM_CSAVRInputCD4DistAtDiagTag : np.full((CSAVR_CD4_500Plus + 1, projInfo['finalYrIdx'] + 1), DPNotAvail).tolist(),
        AM_CSAVRPopulationValueTag : DP_FitToTotal,
        AM_AIDSMortalityAllAgesTag : AIDSMortalityAllAges_Init(projInfo),
        AM_AnnualInterruptionRateTag : 5,
        AM_IncreasedLikelihoodOfReinitTag : 1,
        AM_OffARTMortRateMultiplierTag : 1,
        AM_ChildAnnualInterruptionRateTag : 5,
        AM_ChildIncreasedLikelihoodOfReinitTag : 1,
        AM_ChildOffARTMortRateMultiplierTag : 1,
        AM_CSAVRFitOptionsTag : CSAVRFitOptions_Init(),
        AM_CSAVRParametersTag : CSAVRParameters_Init(CSAVRData),
        AM_CSAVRIncScaleParametersTag : CSAVRIncScaleParameters_Init(CSAVRData),
        AM_CSAVRUncertaintyParamsTag : CSAVRUncertaintyParams_Init(),
        AM_CSAVRConstrainPLHIVGTNumARTTag : np.full(DP_CSAVRMaxFit + 1, False).tolist(),
        AM_CSAVRTypeOfFitTag : DP_DoubleLogistic,
        AM_CSAVRAdjustIRRsTag : np.full((DP_CSAVRMaxFit + 1, DP_CSAVR_DistOfHIV + 1), False).tolist(),
        AM_CSAVRModelFitTypesTag : np.full(DP_CSAVRMaxFit + 1, True).tolist(),
        AM_CSAVRMetaDataTag : [getIRRFittingDataDict() for i in GBRange(0, DP_CSAVRMaxFit)],
        AM_CSAVRMeanCD4atDiagnosisTag : np.zeros((DP_CSAVRMaxFit + 1, GB_Female + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(), #MeanCD4atDiagnosis_Init(projInfo),
        AM_CSAVRMeanCD4atDiagnosisByAgeSexTag : np.zeros((DP_CSAVRMaxFit + 1, GB_Female + 1, DP_A50Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(), #MeanCD4atDiagnosisByAgeSex_Init(projInfo),
        AM_CSAVRTimeInfToDiagTag : np.zeros((DP_CSAVRMaxFit + 1, GB_Female + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(), #TimeInfToDiag_Init(projInfo),
        AM_CSAVRPropOfDiagnosedTag : np.zeros((DP_CSAVRMaxFit + 1, GB_Female + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(), #PropOfDiagnosed_Init(projInfo),
        AM_CSAVRPropOfDiagnosedNoARTTag : np.zeros((DP_CSAVRMaxFit + 1, GB_Female + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_CSAVRPropOfDiagnosedByAgeSexCD4Tag : np.zeros((DP_CSAVRMaxFit + 1, GB_Female + 1, DP_A50Plus + 1, CSAVR_CD4_All + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(), #PropOfDiagnosedByAgeSexCD4_Init(projInfo),
        AM_CSAVRNumPLHIVTag : np.zeros((DP_CSAVRMaxFit + 1, GB_Female + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(), #CSAVRNumPLHIV_Init(projInfo),
        AM_CSAVRNumPLHIVByAgeSexCD4Tag : np.zeros((DP_CSAVRMaxFit + 1, GB_Female + 1, DP_A50Plus + 1, CSAVR_CD4_All + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(), #CSAVRNumPLHIVByAgeSexCD4_Init(projInfo),
        AM_CSAVRNumDiagnosedTag : np.zeros((DP_CSAVRMaxFit + 1, GB_Female + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(), #CSAVRNumDiagnosed_Init(projInfo),
        AM_CSAVRNumDiagnosedByAgeSexCD4Tag : np.zeros((DP_CSAVRMaxFit + 1, GB_Female + 1, DP_A50Plus + 1, CSAVR_CD4_All + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(), #CSAVRNumDiagnosedByAgeSexCD4_Init(projInfo),
        AM_CSAVRAIDSDeathsTag : np.zeros((DP_CSAVRMaxFit + 1, GB_Female + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(), #CSAVRAIDSDeaths_Init(projInfo),
        AM_CSAVRAIDSDeathsByAgeSexTag : np.zeros((DP_CSAVRMaxFit + 1, GB_Female + 1, DP_A50Plus + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(), #CSAVRAIDSDeathsByAgeSex_Init(projInfo),
        AM_CSAVRNumNewInfectionsTag : np.zeros((DP_CSAVRMaxFit + 1, GB_Female + 1, DP_UpperBound + 1, projInfo['finalYrIdx'] + 1)).tolist(), #CSAVRNumNewInfections_Init(projInfo),
        AM_CSAVRIncidenceByFitTag : np.zeros((DP_CSAVRMaxFit + 1, projInfo['finalYrIdx'] + 1)).tolist(),
        AM_HIVSexRatioTag : HIVSexRatio_Init(projInfo, easyAIMData, AMGlobalData),
        AM_HIVSexRatio_MetaTag : {'default': HIVSexRatio_Meta_Init(projInfo, AMGlobalData)},
        AM_CSAVRHIVSexRatioTag : CSAVRHIVSexRatio_Init(projInfo, easyAIMData, AMGlobalData),
        AM_CustomSAPDataTag : CustomSAPData_Init(projInfo, easyAIMData, AMGlobalData),
        AM_CustomSAPData_MetaTag : CustomSAPData_Meta_Init(projInfo),
        AM_CustomSAPDataIndexTag : projInfo['defaultEpidemic'],
        AM_DistOfHIVTag : DistOfHIV_Init(projInfo, AMGlobalData),
        AM_DistOfHIV_MetaTag : {'default': DistOfHIV_Meta_Init(projInfo, AMGlobalData)},
        AM_CSAVRDistOfHIVTag : CSAVRDistOfHIV_Init(projInfo, AMGlobalData),
        AM_EPPANCRTDataTag : EPPANCRTData_Init(projInfo),
        AM_SAPFormDHSFitSurveyTag : SAPFormDHSFitData_Init(projInfo),
        AM_AIDS45q15Tag : getList_year(projInfo['finalYrIdx']),
        AM_NonAIDS45q15Tag : getList_year(projInfo['finalYrIdx']),
        AM_Total45q15Tag : Total45q15_Init(projInfo, AMCountryData),
        AM_Total45q15_MetaTag : {'default' : Total45q15_Meta_Init(projInfo, AMCountryData)},
        AM_Under5MortRateTag : getList_year(projInfo['finalYrIdx']),
        AM_PMTCTProgEstNeedTag : getList_year(projInfo['finalYrIdx']),
        AM_NumberOnARTTag : getList_year(projInfo['finalYrIdx']),
        AM_ARTCovByAgeTag : getList_sex_5yrAge_year(projInfo['finalYrIdx'], DPNotAvail),
        AM_KeyPopsTag : KeyPops_Init(),
        AM_Meningitis_Tag : Meningitis_Init(projInfo, AMCountryData),
        AM_AdvOptsMeningitis_Tag : AdvOptsMeningitis_Init(projInfo, AMCountryData),

        AM_PregWomenPrevRoutineTestTag : 0.0,
        AM_PregWomenPrevTag : PregWomenPrev_Init(projInfo, AMCountryData),
        AM_PregWomenPrev_MetaTag : {'default' : PregWomenPrev_Meta_Init(projInfo, AMCountryData)},
        
        AM_PrevSurveyTag : PrevSurvey_Init(projInfo),
        AM_PrevSurvey_MetaTag : PrevSurvey_Meta_Init(projInfo, AMCountryData, AMSubnatData),
        AM_ARTCovSurveyTag : ARTCovSurvey_Init(projInfo),
        AM_ARTCovSurvey_MetaTag : ARTCovSurvey_Meta_Init(projInfo, AMCountryData),
        AM_MortRateByAgeTag : getList_sex_5yrAge_year(projInfo['finalYrIdx']),
        AM_AllCauseMortalityTag : getList_sex_5yrAge_year(projInfo['finalYrIdx']),
        AM_AllCauseMortality_MetaTag : AllCauseMortality_Meta_Init(projInfo, AMCountryData),
        AM_AIDSMortalityTag : getList_sex_5yrAge_year(projInfo['finalYrIdx']),
        AM_AIDSMortality_MetaTag : AIDSMortality_Meta_Init(projInfo, AMCountryData),
        AM_NumberOnARTByAgeTag : getList_sex_5yrAge_year(projInfo['finalYrIdx']),
        AM_NewlyStartingARTTag : getList_sex_5yrAge_year(projInfo['finalYrIdx']), 
        AM_AdultsChildrenStartingARTTag : getList_sex_5yrAge_year(projInfo['finalYrIdx']), 
        AM_PercentOfPopTag : PercentOfPop_Init(projInfo),
        AM_ARTCoverageSelectionTag : DP_NumOrPercent,
        AM_BFYearsRGIdxTag : DP_DataAllYears,
        AM_BFArvRGIdxTag : (DP_NotInPMTCT -1),
        AM_ChildHIVMortARTRegionTag : projInfo['region'],
        AM_ChildHIVMortARTCustomFlagTag : False,
        AM_ChildARTDistRegionTag : projInfo['region'],
        AM_ChildARTDistCustomFlagTag : False,
        AM_AdultProgressRatesRegionTag : projInfo['region'],
        AM_AdultProgressRatesCustomFlagTag : False,
        AM_AdultHIVMortNoARTRegionTag : projInfo['region'],
        AM_AdultHIVMortNoARTCustomFlagTag : False,
        AM_AdultHIVMortARTRegionTag : projInfo['region'],
        AM_AdultHIVMortARTCustomFlagTag : False,
        AM_DALYBaseYearTag : max(2015, projInfo['firstYr']),
        AM_DALYDiscountRateTag : 3.0,
        AM_DALYUseStandardLifeTableTag : False,
        AM_OrphansRegionalPatternTag : DP_SSA,
        AM_IncidenceInput1970BoolTag : False,
        AM_IncidenceOptionsTag : DP_DirectIncidenceInputOpt,
        AM_IncidenceByFitTag : incidenceByFit,
        AM_FourDecPlaceIDTag : False,
        AM_EPPPrevAdjTag : True,
        AM_EPPMaxAdjFactorTag : 10,
        AM_EPPPopulationAgesTag : DP_EPP_15t49,
        # AM_IncEpidemicRGIdxTag : 0,
        AM_IncEpidemicCustomFlagTag : False,
        AM_SexRatioFromEPPTag : False,
        AM_HIVSexRatioFromEPPTag : getList_year(projInfo['finalYrIdx']),
        AM_EpidemicTypeFromEPPTag : DP_EPP_GENERALIZED,
        AM_AdultPrevalenceTag : AdultPrevalence_Init(projInfo, easyAIMData),
        AM_EPPCountryNameTag : '',
        AM_EPPEpiNameTag : EPPEpiName_Init(),
        AM_EPPEpidemicTag : EPPEpidemic_Init(),
        AM_EPPPrevDataTag : EPPPrevData_Init(projInfo),
        AM_EPPIncDataTag : EPPIncData_Init(projInfo),
        AM_EPPPopDataTag : EPPPopData_Init(projInfo),
        AM_EPPSexRatioTag : EPPSexRatio_Init(projInfo),
        AM_EPPBaseYrPopTag : EPPBaseYrPop_Init(),
        AM_EPPIDUMortalityTag : EPPIDUMortality_Init(),
        AM_EPPPathInfoTag : EPPPathInfo_Init(),
        AM_EPPAgeRangeTag : 0,
        AM_NumOfEPPEpidemicsTag : 0,
        AM_YrPtPrevalence_WBTag : YrPtPrevalence_WB_Init(),
        AM_AIDSDeathsAmongIDUTag : getList_year(projInfo['finalYrIdx']),
        AM_PropIDU_WBTag : PropIDU_WB_Init(projInfo),
        AM_NonAIDSDeathsAmongIDUTag : getList_year(projInfo['finalYrIdx']),
        AM_ChAged14ByCD4CatTag : ChAged14ByCD4Cat_Init(projInfo), 
        AM_NewHIVInfByRiskGroupTag  : [],
        # AM_IMRTag : getList_sex_year(projInfo['finalYrIdx']),  
        # AM_CMRTag : getList_sex_year(projInfo['finalYrIdx']),
        AM_HIVExpUninfChildrenTag : getList_sex_year(projInfo['finalYrIdx']), 
        AM_ARTExpUninfChildrenTag : getList_sex_year(projInfo['finalYrIdx']),
        AM_NewInfantInfectionsTag : getList_year(projInfo['finalYrIdx']), 
        AM_CD4DistributionTag : CD4Distribution_Init(projInfo), 
        AM_CD4Distribution15_49Tag : CD4Distribution15_49_Init(projInfo), 
        AM_CD4DistributionChildTag : CD4DistributionChild_Init(projInfo), 
        AM_DeathsByAgeTag : getList_sex_singleAge_year(projInfo['finalYrIdx']), 
        AM_HIVBySingleAgeTag : getList_sex_singleAge_year(projInfo['finalYrIdx']), 


        AM_RegionalOutputTag : [],#np.zeros((DP_AdPop0_14Female_Child + 1, DP_MaxEpidemic + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_OrphansTag : np.zeros((DP_Paternal_Orphans + 1, DP_MaxChildAge + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_AgeLastBDayTag : np.zeros((DP_Dual_ALB + 1, DP_MaxChildAge + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_PopAdjTag : getList_sex_year(projInfo['finalYrIdx']), 
        AM_PopAdjAmountTag : getList_sex_5yrAge_year(projInfo['finalYrIdx']), 
        AM_AIDSDeathsByAgeTag : getList_sex_singleAge_year(projInfo['finalYrIdx']), 
        AM_AIDSDeathsARTSingleAgeTag : getList_sex_singleAge_year(projInfo['finalYrIdx']), 
        AM_AIDSDeathsNoARTSingleAgeTag : getList_sex_singleAge_year(projInfo['finalYrIdx']), 
        AM_HIVPregWomenTag : getList_year(projInfo['finalYrIdx']), 
        AM_HIVPregWomenCD4LT350Tag : getList_year(projInfo['finalYrIdx']), 
        AM_ChildOnCotrimTag : getList_sex_year(projInfo['finalYrIdx']), 
        AM_ChildNeedCotrimTag : getList_sex_year(projInfo['finalYrIdx']), 
        AM_NeedARTTag : getList_sex_singleAge_year(projInfo['finalYrIdx']), 
        AM_UnmetNeedARTTag : getList_sex_5yrAge_year(projInfo['finalYrIdx']), 
        AM_PerinatalTransmissionTag : getList_year(projInfo['finalYrIdx']), 
        AM_ChildCTXNeed1To4Tag : getList_sex_year(projInfo['finalYrIdx']), 
        AM_TotalOrphansTag : np.zeros((DP_TotalOrphans + 1, DP_Non_AIDS + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_AllAIDSOrphTag : np.zeros((DP_AllAIDSOrphans + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_AllAIDSNewOrphTag : np.zeros((DP_AllAIDSOrphans + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_NewlyNeedARTTag : getList_sex_5yrAge_year(projInfo['finalYrIdx']), 
        AM_ChildNeedPMTCTTag : getList_year(projInfo['finalYrIdx']), 
        AM_ChildOnPMTCTTag : getList_year(projInfo['finalYrIdx']), 
        AM_MTCTRate6WksTag : getList_year(projInfo['finalYrIdx']), 
        AM_OnARTBySingleAgeTag : getList_sex_singleAge_year(projInfo['finalYrIdx']), 
        AM_OnARTBySingleAge_ProgDataTag : getList_sex_singleAge_year(projInfo['finalYrIdx']), 
        AM_NewInfectionsBySingleAgeTag : getList_sex_singleAge_year(projInfo['finalYrIdx']), 
        AM_NewlyOnARTTag : np.zeros(((10) + 1, GB_Female + 1, GB_MAX_AGE + 1, DP_H_Max + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_ViralSupprPhi_TempTag : ViralSupprPhi_Init(AMGlobalData),
        AM_SexuallyActive15to19Tag : SexuallyActive15to19_Init(AMCountryData), 
        AM_SexuallyActive15to19_MetaTag : {'default' : SexuallyActive15to19_Init(AMCountryData)}, 
        AM_ReadFRR15_19Coefs_TempTag : ReadFRR15_19Coefs_Init(projInfo, AMGlobalData),
        AM_TFR15_19DefaultDataTag : TFR15_19DefaultData_Init(projInfo, AMCountryData, AMGlobalData),
        AM_FRRFitXMLDefaultDataTag : FRRFitXMLDefaultData_Init(projInfo),
        AM_HAARTBySexPerNumTag : HAARTBySexPerNum_Init(projInfo, easyAIMData),
        AM_AdultARTAdjFactorTag : getList_sex_year(projInfo['finalYrIdx'], 1),
        AM_AdultARTAdjFactorFlagTag : False,
        AM_AdultPatsAllocToFromOtherRegionTag : getList_sex_year(projInfo['finalYrIdx']),
        AM_KnowledgeOfStatusFileTitleTag : '',
        AM_CascadeActiveTag :  False,
        AM_PMTCTEffRegTag : getList_year(projInfo['finalYrIdx']), 
        AM_HIVPosBFWomen3MonthsTag : getList_year(projInfo['finalYrIdx']), 
        AM_HIVPosBFWomen12MonthsTag : getList_year(projInfo['finalYrIdx']), 
        AM_AIDSDeathsNoIntervTag : np.zeros((4 + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_DeathsAvertARTTag : getList_year(projInfo['finalYrIdx']), 
        AM_InfectAvertPMTCTTag : getList_year(projInfo['finalYrIdx']), 
        AM_LifeYearsGainedTag : getList_year(projInfo['finalYrIdx']), 
        AM_ChildDeathsAvertARTTag : np.zeros((4 + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_DeathsAvertCotrimTag : np.zeros((4 + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_DeathsAvertPMTCTTag : np.zeros((4 + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_HIVAgingRateTag : np.zeros((GB_Female + 1, GB_MAX_AGE + 1, DP_HIVARTgt12m + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_HIVChildNoARTByInfectionTag : np.zeros((GB_Female + 1, 14 + 1, DP_P_BF12 + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_TreatPercentTag : np.zeros((DP_OptB_BF + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_MedianCD4Tag : getList_year(projInfo['finalYrIdx']), 
        AM_AIDSMortAllAgesFYrAdjIdxTag : 0,
        
        AM_ValidationNumberOnARTByAgeTag : ValidationControlMV_Init(projInfo),
        AM_ValidationNewlyOnARTTag : ValidationControlMV_Init(projInfo),
        AM_ValidationMortalityRateByAgeTag : ValidationControlMV_Init(projInfo),
        AM_ValidationAIDSMortalityByAgeTag : ValidationControlMV_Init(projInfo),
        AM_ValidationAllCausesMortalityByAgeTag : ValidationControlMV_Init(projInfo),
        AM_ValidationPrevalenceSelectionTag : PrevalenceValidationControlMV_Init(projInfo),
        AM_ValidationARTCoverageSelectionTag : ValidationARTCovSelection_Init(),
        AM_ValidationAllCauseDeathsARTTag : getList_year(projInfo['finalYrIdx']),
        AM_ValidationWaterfallTag : ValidationWaterfall_Init(projInfo),
        
        AM_PercART50PlusTag : getList_sex_year(projInfo['finalYrIdx']), 
        AM_Prop350_500Tag : 0, 
        AM_IncidenceAdjustmentFactorTag : getList_year(projInfo['finalYrIdx']), 
        AM_ECDCFQNameTag : '',
        AM_NewARTPatAllocTag : NewARTPatAlloc_Init(easyAIMData),
        AM_NewARTPatAlloc_MetaTag : {'default' : NewARTPatAlloc_Init(easyAIMData)},       
        AM_NewARTPatAllocMethodTag : NewARTPatAllocMethod_Init(easyAIMData),
        AM_DefaultEpidemicTag : projInfo['defaultEpidemic'],
        AM_FirstYearOfEpidemicTag : projInfo['epidemicStartYr'],        
        
        AM_ResNonAIDSDeathsHIVPopNoARTTag : getList_sex_singleAge_year(projInfo['finalYrIdx']),
        AM_ResMTCTBySourceTag : np.zeros((PMTCT_TransmissionRate + 1, max(MTCT_Ind_NumWomen_MaxInd, MTCT_Ind_MaxInd) + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_ResDetailedTreatmentCascadeTag : np.zeros((DP_Cas_ViralSuppr + 1,  max(max(DP_DetCas_KoSIncidentInfNotDiag, DP_DetCas_TreatStatusDiagNoTreat), DP_DetCas_ViralSupprNotSuppressedOnTreat) + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_ResTreatmentCascadeTag : np.zeros((DP_Cas_AG_AdultBothSexes + 1, DP_Cas_IND_PerPLHIV + 1, DP_Cas_ViralSuppr + 1, projInfo['finalYrIdx'] + 1)).tolist(), 
        AM_ResPerinatallyInf15_19Tag : getList_sex_year(projInfo['finalYrIdx']),
        AM_ResPerinatallyInf20_24Tag : getList_sex_year(projInfo['finalYrIdx']),
        AM_ResQALYsTag : getList_year(projInfo['finalYrIdx']),
        AM_ResDALYsAIDSOnlyTag : getList_year(projInfo['finalYrIdx']),
        AM_ResDALYsTag : getList_year(projInfo['finalYrIdx']),
        AM_ResYLDTag : getList_year(projInfo['finalYrIdx']),
        AM_ResNonAIDSDeathsHIVPopARTTag : getList_sex_singleAge_year(projInfo['finalYrIdx']),
        AM_ResTotalDeathsHIVPopTag : getList_sex_singleAge_year(projInfo['finalYrIdx']),
        AM_ResAIDS45q15Tag : getList_year(projInfo['finalYrIdx']),
        AM_ResNonAIDS45q15Tag : getList_year(projInfo['finalYrIdx']),
        AM_ResNumWithAHDTag : getList_sex_year(projInfo['finalYrIdx']),                     
        AM_ResNumWithAHDOnARTTag : getList_sex_year(projInfo['finalYrIdx']),               
        AM_ResNumWithAHDNoARTTag : getList_sex_year(projInfo['finalYrIdx']),                
        AM_Res15PlusNumWithCryptoAntigenemiaTag : getList_sex_year(projInfo['finalYrIdx']), 
        AM_Res15PlusNumWithCryptoMeningitisTag : getList_sex_year(projInfo['finalYrIdx']),  
        AM_Res15PlusDeathsToPLHIVFromCMTag : getList_sex_year(projInfo['finalYrIdx']),      
        AM_Res15PlusPercAIDSDeathsFromCMTag : getList_sex_year(projInfo['finalYrIdx']),   
        AM_ResOnARTgt1Yr15PlusTag : getList_sex_year(projInfo['finalYrIdx']),     
        AM_ResOnARTlt1Yr15PlusTag : getList_sex_year(projInfo['finalYrIdx']),     
        
        AM_ChildARTAdjFactorTag : np.ones((projInfo['finalYrIdx'] + 1)).tolist(),
        AM_ChildARTAdjFactorFlagTag : False,
        AM_ChildPatsAllocToFromOtherRegionTag : getList_year(projInfo['finalYrIdx']),
        
        AM_SubnatPrevalenceTag       : {}, #Created in AMNaomiSubnational.py
        AM_SubnatARTCoverageTag      : {}, #Created in AMNaomiSubnational.py
        AM_SubnatCustomBaseYrFlagTag : False,
        
        # AM_PreventionNeedsFirstTimeTag   : True,
        AM_PreventionNeedsShowVMMCTabTag : 'VMMC'in PreventionNeedsDB,
        AM_PreventionNeedsCondomsTag     : ReadPreventionNeeds_Condoms(PreventionNeedsDB),
        AM_PreventionNeedsVMMCTag        : ReadPreventionNeeds_VMMC(PreventionNeedsDB),
        AM_PreventionNeedsKeyPopsTag     : ReadPreventionNeeds_KeyPopulations(PreventionNeedsDB),
        AM_PreventionNeedsPrEPTag        : ReadPreventionNeeds_PrEP(PreventionNeedsDB),
        # AM_PrEPParametersTag             : [0.0]*5,
        
        AM_PMTCTPrEPParametersTag    : {
            "adherence": {
                "oral": 0.75,
                "injectable": 0.9
            },
            "selection": {
                # Ratio of incidence among those receiving PrEP to all pregnant and breastfeeding women
                "incidenceRatio": 1.0
            },
            # Person-years of PrEP per person receiving any PrEP during pregnancy and breastfeeding
            "personYearsPrEP": { 
                "oral": 0.59, 
                "injectable": 0.85
            }
        },

        AM_PMTCTReceivingOralPrEPTag       : getList_year(projInfo['finalYrIdx']),
        AM_PMTCTReceivingInjectablePrEPTag : getList_year(projInfo['finalYrIdx']),

    }

    return MVs # only put this here to aid debugging